#include "GrabCutTest.h"

#include <future>
#include <utility>

#include <opencv2/core/core.hpp>
#include <opencv2/highgui/highgui.hpp>
#include <opencv2/imgproc/imgproc.hpp>
#include "opencv2/imgproc/types_c.h"
#include <iostream>

#include "grabcut/MyGrabCut.hpp"
#include "grabcut/MyGrabCutD.hpp"

//#include "GCoptimization.h"

using namespace std;

namespace am
{
    void GrabCutTest::run( GrabcutResult& result, t_annotated_input const& input /*, t_snakes const& snakes, cv::Mat const& mask, cv::Mat const& img, cv::Mat const& dep, cv::Mat const& dep8*/ )
    {
        using namespace cv;
        using namespace std;

        // compress channels, so that depth fits
        cv::Mat img_preprocessed;
        cv::cvtColor( input.img, img_preprocessed, CV_RGB2HSV );
        if ( !input.dep.empty() )
        {
            for ( unsigned y = 0; y < img_preprocessed.rows; ++y  )
            {
                for ( unsigned x = 0; x < img_preprocessed.cols; ++x )
                {
                    //img2.at<uchar>(y,x*3+1) = ((float)img.at<uchar>(y,x*3) +(float)img.at<uchar>(y,x*3+1) + (float)img.at<uchar>(y,x*3+2)) / 3.f;

                    // replace saturation with depth
                    img_preprocessed.at<uchar>(y,x*3+1) = input.dep8.at<uchar>( y, x );
                }
            }
            cv::imshow("img2", img_preprocessed );
        }

        // tmp vars
        cv::Mat bgModel, fgModel; // the models (internally used)

        // GrabCut segmentation
        std::cout << "starting..." << std::endl;

        // backup user annotationn
        cv::Mat user_mask = input.mask.clone();

        {
            // GrabCut segmentation
            am::MyGrabCut::grabCut<3,cv::Vec3b>( img_preprocessed,         // input img
                                      input.mask,                     // segmentation result
                                      cv::Rect(),               // rectangle containing foreground
                                      bgModel,fgModel,          // models
                                      1,                        // number of iterations
                                      cv::GC_INIT_WITH_MASK );  // use rectangle
            // show
            GrabCutTest::showMask( input.mask, input.img, "Segmentation with depth" );
        }

        {
            bgModel = fgModel = cv::Mat();
            // restore user annotation
            user_mask.copyTo( input.mask );

            // GrabCut segmentation
            am::MyGrabCut::grabCut<3,cv::Vec3b>( input.img,                      // input img
                                      input.mask,                     // segmentation result
                                      cv::Rect(),               // rectangle containing foreground
                                      bgModel, fgModel,         // models
                                      1,                        // number of iterations
                                      cv::GC_INIT_WITH_MASK );  // use rectangle
            // show
            GrabCutTest::showMask( input.mask, input.img, "Segmentation without depth" );
        }

        {
            bgModel = fgModel = cv::Mat();

            // restore user annotation
            user_mask.copyTo( input.mask );

            // GrabCut segmentation
            am::MyGrabCut::grabCut<1,uchar>( input.dep8,                     // input img
                                  input.mask,                     // segmentation result
                                  cv::Rect(),               // rectangle containing foreground
                                  bgModel, fgModel,         // models
                                  1,                        // number of iterations
                                  cv::GC_INIT_WITH_MASK );  // use rectangle
            // show
            GrabCutTest::showMask( input.mask, input.img, "Segmentation ONLY depth" );
        }

        std::cout << "gc finished..." << std::endl;
        char key = 0;
        while ( (key = waitKey()) != 27 ) ;
        //return 0;
    }

    void GrabCutDTest::run( GrabcutResult& result, t_annotated_input const& input /*t_snakes const& snakes, cv::Mat const& mask_arg, cv::Mat const& img, cv::Mat const& dep, cv::Mat const& dep8 */ )
    {
        using namespace cv;
        using namespace std;

        // GrabCut segmentation
        std::cout << "GrabCutDTest starting..." << std::endl;

        // backup user annotationn
        const cv::Mat user_mask = input.mask.clone();

        // find maximum label in the input
        uchar max_label = 0;
        {
            for ( auto const& snake : input.snakes )
            {
                if ( snake[0].z > max_label )
                {
                    max_label = snake[0].z;
                }
            }
            std::cout << "max_label is: " << (int)max_label << std::endl;
        }

        // if betas empty, the default to 1-s, and we can change them
        // if betas filled, only one iteration is run with the copied betas
        am::MyGrabCutD::params_struct params;
        if ( input.betas.size() > 0 )
        {
            params.beta_coeffs[0] = input.betas[0];
            params.beta_coeffs[1] = input.betas[1];
        }

        result.empty();
        result.set_image( input.img );
        result.set_depth( input.dep );
        t_snakes const& snakes = input.snakes;
        cv::Mat  const& img    = input.img;
        cv::Mat  const& dep    = input.dep;

        char key = 0;
        while ( key != 27 )
        {
            // reset key
            key = 0;

            std::vector< std::future<std::pair<cv::Mat,uchar>> > vecFutures;

            // make segmentation for every label
            for ( uchar label = 0; label <= max_label; ++label )
            {
                // for every label, start a lambda function returning a mat and a uchar
                vecFutures.emplace_back( std::async( std::launch::async, [label, &snakes, &user_mask, &img, &dep, &params]()
                {
                    std::cout << "starting graphcut label: " << (int)label << std::endl;

                    cv::Mat mask( user_mask.size(), CV_8UC1 );
                    // prepare mask
                    {
                        mask.setTo( GC_PR_BGD );
                        for ( auto const& snake : snakes )
                        {
                            for ( int snake_id = 1; snake_id < snake.size(); ++snake_id )
                            {
                                // colour fgd, if it is the current label
                                // colour bgd, if it is another label
                                cv::line( mask,                                                     // target image
                                          cv::Point(snake[snake_id-1].x,snake[snake_id-1].y),       // start point
                                          cv::Point(snake[snake_id  ].x,snake[snake_id  ].y),       // end point
                                          (snake[0].z == label ?                                    // colour
                                              cv::Scalar( cv::GC_FGD ) : cv::Scalar( cv::GC_BGD )),
                                          4                                                         // thickness
                                        );
                            }
                        }
                    }

                    // prepare images
                    std::vector<cv::Mat> images = { img, dep };

#if 1
                    // prepare models
                    std::vector<cv::Mat> models(4);

                    // GrabCut segmentation
                    am::MyGrabCutD::grabCutD<cv::Vec3b,ushort>( images,                   // input img
                                                                mask,                     // segmentation result
                                                                cv::Rect(),               // unused: rectangle containing foreground
                                                                models,                   // models
                                                                1,                        // number of iterations
                                                                cv::GC_INIT_WITH_MASK,    // use rectangle
                                                                &params );
#else
                    // prepare models
                    std::vector<cv::Mat> models(2);

                    // GrabCut segmentation
                    am::MyGrabCutD::grabCutD4<cv::Vec3b,ushort>( images,                   // input img
                                                                mask,                     // segmentation result
                                                                cv::Rect(),               // unused: rectangle containing foreground
                                                                models,                   // models
                                                                1   ,                        // number of iterations
                                                                cv::GC_INIT_WITH_MASK,    // use rectangle
                                                                &params );
#endif

                    std::cout << "GrabCutDTest finished for label..." << (int)label << std::endl;

                    // show
                    {
                        char title[256];
                        sprintf( title, "LABEL%d: Segmentation with depth", (int)label );
                        GrabCutTest::showMask( mask, img, title, 640 + label * 100, 10 );
                    }

                    return std::pair<cv::Mat,uchar>( mask, label );
                })); // async
            } // for labels

            // collect assync results
            for ( auto &future : vecFutures )
            {
                auto mask_with_label = future.get();
                result.addForLabel( mask_with_label.first, mask_with_label.second );
            }

            if ( input.betas.size() )
                break;

            // adjust beta
            while ( (key != 10) && (key != 27) )
            {
                key = cv::waitKey();
                std::cout << "key: " << (int)key << std::endl;
                switch ( key )
                {
                    case '1':
                        params.beta_coeffs[0] += .1;
                        break;
                    case '2':
                        params.beta_coeffs[0] -= .1;
                        break;
                    case '3':
                        params.beta_coeffs[1] += .1;
                        break;
                    case '4':
                        params.beta_coeffs[1] -= .1;
                        break;

                }
                std::cout << "beta_coeffs: " << params.beta_coeffs[0] << ", " << params.beta_coeffs[1] << std::endl;
            }
        }
    }

} // ns am
