#include "GrabCutTestMultiLabel.h"

#include "GCoptimization.h"
#include "MultiLabelGrabCut.h"
#include "MultiLabelGrabCut.hpp"

#include <iostream>
#include <opencv2/imgproc/imgproc.hpp>

namespace am
{
    void GrabCutTestMultiLabel::run( GrabcutResult& result,
                                     t_snakes const& snakes, cv::Mat const& mask,
                                     cv::Mat const& img, cv::Mat const& dep, cv::Mat const& dep8 )
    {
        // GrabCut segmentation
        std::cout << "starting GrabCutTestMultiLabel::run()..." << std::endl;

        // backup user annotationn
        cv::Mat user_mask = mask.clone();

        std::vector<cv::Mat> input_images = { img, dep };
        std::vector<cv::Mat> masks        = { mask, mask };
        std::vector<cv::Mat> models(2);
        std::vector<cv::Mat> results;

        {
            // GrabCut segmentation
            am::MultiLabelGrabCut::grabCut( input_images,         // input img
                                            masks,                 // segmentation result
                                            models,
                                            results,
                                            1,                        // number of iterations
                                            cv::GC_INIT_WITH_MASK );  // use rectangle
            // show
            MyGrabCutEnv::showMask( mask, img, "Segmentation with depth" );
        }
    }

#if 0
    // setup multilabel opt

    int width = 10;
    int height = 5;
    int num_pixels = width*height;
    int num_labels = 7;

    int *result = new int[num_pixels];   // stores result of optimization

    try
{
    GCoptimizationGridGraph *gc = new GCoptimizationGridGraph( width, height, num_labels );

    // first set up data costs individually
    for ( int i = 0; i < num_pixels; i++ )
    for (int l = 0; l < num_labels; l++ )
    if (i < 25 )
    {
        if(  l == 0 ) gc->setDataCost(i,l,0);
        else gc->setDataCost(i,l,10);
    }
    else {
    if(  l == 5 ) gc->setDataCost(i,l,0);
    else gc->setDataCost(i,l,10);
}

// next set up smoothness costs individually
for ( int l1 = 0; l1 < num_labels; l1++ )
{
    for (int l2 = 0; l2 < num_labels; l2++ )
    {
        int cost = (l1-l2)*(l1-l2) <= 4  ? (l1-l2)*(l1-l2):4;
        gc->setSmoothCost(l1,l2,cost);
    }
}

printf("\nBefore optimization energy is %d",(int)gc->compute_energy());
gc->expansion(2);// run expansion for 2 iterations. For swap use gc->swap(num_iterations);
printf("\nAfter optimization energy is %d",(int)gc->compute_energy());

for ( int  i = 0; i < num_pixels; i++ )
result[i] = gc->whatLabel(i);

delete gc;
}
catch (GCException e)
{
    e.Report();
}

delete [] result;
}
#endif
}
