#ifndef CUBOIDUTIL_H
#define CUBOIDUTIL_H

#include <vector>
#include <string>
#include "opencv2/core/core.hpp"
#include "boost/filesystem.hpp"
#include "cuboid_approx/primitives/Contact.h"

#include <iostream>

namespace am
{

    class CuboidUtil
    {
        public:
            static cv::Mat
            loadDepth( std::string depth_path );

    };

    namespace util
    {
        // return the filenames of all files that have the specified extension
        // in the specified directory and all subdirectories
        inline void get_files_by_ext( std::vector<std::string>                    & ret
                                      , ::boost::filesystem::path            const& root
                                      , std::string                                 ext )
        {
            if ( ext[0] != '.' )
            {
                std::cout << "[" << __func__ << "]: " << "prepending \".\" to ext " << ext;
                ext = "." + ext;
                std::cout << " --> " << ext << std::endl;
            }

            if (!::boost::filesystem::exists(root)) return;

            if ( ::boost::filesystem::is_directory(root) )
            {
                ::boost::filesystem::recursive_directory_iterator it( root );
                ::boost::filesystem::recursive_directory_iterator endit;
                while ( it != endit )
                {
                    //std::cout << "file: " << *it;
                    if (::boost::filesystem::is_regular_file(*it) && (it->path().extension() == ext) )
                    {
                        ret.push_back(it->path().string());
                    }
//                    else
//                    {
//                        if ( !::boost::filesystem::is_regular_file(*it) )
//                        {
//                            std::cout << ", not regular";
//                        }
//                        if ( it->path().extension() != ext )
//                        {
//                            std::cout << ", not matching ext: " << it->path().extension();
//                        }
//                    }
//                    std::cout << std::endl;
                    ++it;
                }
            }
        }

        inline std::string
        cuboidPathToWallsPath( boost::filesystem::path cuboid_path )
        {
            return (cuboid_path.parent_path() / cuboid_path.stem()).string() + ".walls";
        }

        inline int
        saveGTContacts( std::vector<am::Contact> const& contacts, std::string const& str_gt_path )
        {
            std::string out_path = str_gt_path.substr( 0, str_gt_path.rfind(".") ) + "_graph.txt";
            if ( boost::filesystem::exists( out_path ) )
            {
                out_path += ".new";
            }
            if ( !boost::filesystem::exists( out_path ) )
            {
                am::Contact::write( contacts, out_path );
                std::cout << "[" << __func__ << "]: " << "contacts written to " << out_path << std::endl;
            }

            return EXIT_SUCCESS;
        }
    }

} // nsam

#endif // CUBOIDUTIL_H
