#ifndef MYGRABCUT_H
#define MYGRABCUT_H

#include <string>
#include <vector>
#include <opencv2/core/core.hpp>
#include "GrabcutResult.h"

#include "Params.h"

namespace am
{
    typedef std::vector<std::vector<cv::Point3i> > t_snakes;

    struct t_annotated_input
    {
            t_snakes snakes;
            cv::Mat mask, img, dep, dep8;

            std::vector<double> betas;
    };

    class MyGrabCutEnv
    {
        public:
            /// STATIC METHODS
            // annotation
            static int loadAndAnnotate( t_snakes &snakes, cv::Mat &mask, cv::Mat &img_, cv::Mat &dep_,
                                         const std::string &path, const std::string &depth_path = "", cv::Mat *p_dep8 = nullptr, bool cheat = false );
            // display
            static void showMask(const cv::Mat& mask, const cv::Mat &img_,
                                 const std::string title = "Segmented img", int x = -1, int y = -1);

            /// CONSTR

            /// METHODS
            int
            destroyWindows();

            /// VIRTUAL METHODS
            // entry points
            virtual int run( GrabcutResult& result, std::string const& path, std::string const depth_path = "", bool cheat = false, am::Params *p_params = nullptr );
            virtual int run( GrabcutResult &result, cv::Mat const& img, cv::Mat const& dep, bool cheat, Params *p_params );

            // logic to override
            virtual void run( GrabcutResult& result, t_annotated_input const& input );
            //virtual void run( GrabcutResult& result, t_snakes const& snakes, cv::Mat const& mask, cv::Mat const& img_, cv::Mat const& dep_, cv::Mat const& dep8 );

            /// SET, GET
            t_annotated_input getAnnotatedInput();
            t_snakes const&   getSnakes();
            cv::Mat  const&   getMask();
            cv::Mat  const&   getImg();
            cv::Mat  const&   getDep();
            cv::Mat  const&   getDep8();

        protected:
            static int
            _annotate( t_snakes &points, cv::Mat &mask, cv::Mat const& img, cv::Mat const& dep );

            t_snakes m_snakes;
            cv::Mat  m_mask;
            cv::Mat  m_img;
            cv::Mat  m_dep, m_dep8;

            static const std::string win_name_rgb;
            static const std::string win_name_dep;
    };



}

#endif // MYGRABCUT_H
