#ifndef PARAMS_H
#define PARAMS_H

#include <string>
#include <map>

namespace am
{

    class IntParam;
    class StringParam;
    class DoubleParam;

    class Param
    {
        public:
            IntParam*    asIntParam();
            StringParam* asStringParam();
            DoubleParam* asDoubleParam();
    };

    class IntParam : public Param
    {
        public:
            IntParam( int value );
            int _value;
    };

    class DoubleParam : public Param
    {
        public:
            DoubleParam( double value );
            double _value;
    };

    class StringParam : public Param
    {
        public:
            StringParam( std::string value );
            std::string _value;
    };

    class Params
    {
        public:
            bool
            hasKey( std::string key ) const;

            int
            setInt( std::string key, int val );
            int
            getInt( std::string key, int default_value = -1 ) const;

            int
            setString( std::string key, std::string val );
            std::string
            getString( std::string key, std::string default_value = "" ) const;

            int
            setDouble( std::string key, double val );
            double
            getDouble( std::string key, double default_value = -1.f ) const;

            virtual ~Params();

        protected:
            std::map<std::string,Param*> _params;
    };

} // ns am

#endif // PARAMS_H
