#ifndef __CA_ANNOTATIONSTATE_H__
#define __CA_ANNOTATIONSTATE_H__

#include <set>

#include "cuboid_approx/common.h"
#include "cuboid_approx/ui/DisplayedPrimitive.h"

namespace am
{
    class Contact;
}

namespace ca
{
    // inner state of the AnnotationController
    class AnnotationState
    {
        public:
            AnnotationState();

            DisplayedPrimitive::Map                    _objects;
            int                                        _next_primitive_id;
            std::vector<PCloud::Ptr>                   _aux_clouds;

            std::vector<std::string>                   _drawn_contacts;
            std::vector<std::string>                   _drawn_models;

            float                                      _max_sphere_size, _min_sphere_size;
            std::vector<am::Contact>                   _latest_contacts;
            bool                                       _colour_edges, _show_IDs;
            double                                     _display_point_size;

            // TODO:
            enum DIRTY_LEVEL { NONE = 0, COLOUR = 1, FULL = 2 };
            PCloud::Ptr                 & MainCloudRef      ( int change_level );
            PCloud::Ptr            const& MainCloud         ()                   const;

            PCloud::Ptr                 & CurrentCloudRef   ( int change_level );
            PCloud::Ptr            const& CurrentCloud      ()                   const;

            pcl::PointIndices::Ptr      & CurrentIndicesRef ();
            pcl::PointIndices::Ptr const& CurrentIndices    ()                   const;

            int                     addPick             ( int point_id, int obj_id = -1 );
            int                     hasPick             ( int point_id, int obj_id = -1 );
            int                     removePick          ( int point_id, int obj_id = -1 );
            int                     clearPick           ( int obj_id = -1 );
            std::vector<int>        pickedIndices       ( int obj_id = -1 );
            float                   getClosestPickedTo  ( int point_id, int obj_id = -1 );

        protected:
            PCloud::Ptr                                 _main_cloud_ptr;
            PCloud::Ptr                                 _current_cloud_ptr;
            pcl::PointIndices::Ptr                      _current_indices_ptr;

            bool                                        _main_cloud_dirty, _current_cloud_dirty, _current_indices_dirty;
            std::vector<std::set<int> >                 _picked;
    }; // ... class AnnotationState

} // ... ns ca

#endif // __CA_ANNOTATIONSTATE_H__
