#ifndef __CA_CONTACT_H__
#define __CA_CONTACT_H__

#include <vector>
#include "Eigen/Dense"
#include "pcl/visualization/pcl_visualizer.h"

#include "primitives/Cuboid.h"

namespace am
{
    class Contact
    {
        public:
            enum TYPE { TOUCHING, FIXED, UNDEFINED = -1, REMOVED };
            typedef std::vector<Contact> Vec;
            static const double          CONTACT_LINE_WIDTH;

            // CONSTR
            Contact()
                : type( UNDEFINED ) {}

            Contact( std::vector<int> const& ids, TYPE type )
                : ids( ids ), type( type ) {}

            // METHODs
            std::vector<std::string>
            draw( am::Cuboid                               const& cubA
                  , Eigen::Matrix<float,4,1>               const& colourA
                  , am::Cuboid                             const& cubB
                  , Eigen::Matrix<float,4,1>               const& colourB
                  , pcl::visualization::PCLVisualizer::Ptr        viewer
                  , int                                    const  viewport
                  , double                                 const  line_width /*= AnnotationController::CUBOID_SELECTED_LINE_WIDTH*/
                  , float                                  const  max_radius /*= _state._max_sphere_size */
                  , float                                  const  min_radius = .02f ) const;


            // OPERATORs
            bool operator==( Contact const& other ) const;
            bool operator< ( Contact const& other ) const;

            // SET, GET
            Eigen::Vector4f getColour() const;
            std::string     toString()  const;

            // STATICs
            static int
            read( std::vector<am::Contact>      & contacts
                  , std::string            const& fileName
                  , int                         * max_proxy_id_arg  = NULL
                  , bool                   const  silent            = true );
            static int
            write( std::vector<am::Contact> const& contacts
                   , std::string            const& fileName );


            static int
            compareFiles( std::string   const& path
                          , std::string const& path_gt
                          , bool               verbose              = false
                          , float            * p_precision          = NULL
                          , float            * p_recall             = NULL
                          , int              * max_allowed_box_id   = NULL
                          , std::string      * map_path             = NULL
                          , int              * contact_count        = NULL
                          , int              * cuboid_count         = NULL);

            static int
            filterByMaxCuboidID( am::Contact::Vec &filtered
                                 , am::Contact::Vec input
                                 , int max_cuboid_id );

//            static int
//            sort( std::vector<Contact> &contacts );

            // STATIC FIELDs
            static const std::string pattern;
            static const int         number_width;

            // FIELDs
            std::vector<int>         ids;
            TYPE                     type;
    };
} // ... ns am

#endif // __CA_CONTACT_H__
