#ifndef __CA_CONTACTSFILTERING_H__
#define __CA_CONTACTSFILTERING_H__

#include "cuboid_approx/primitives/Contact.h"
#include "primitives/Cuboids.h"

namespace ca
{
    class ContactsFiltering
    {
        public:
            ///
            /// \brief The RELATIVE_POS enum
            /// ABOVE           : centroid higher
            /// ABOVE_TOP_PLANE : centroid higher than top plane
            /// SUPPORT         : ABOVE_TOP_PLANE & centroid projection inside top plane
            ///
            enum RELATIVE_POS { NONE = 0, ABOVE = 1, ABOVE_TOP_PLANE = 3, SUPPORT = 4 };

            ///
            /// \brief centroidAbove Assumes cuboid.Planes() normals are pointing inwards.
            /// \param above cuboid
            /// \param below cuboid
            /// \param up    up_vector to interpret "above"
            /// \return true, if above's centroid above below's "top" plane
            ///
            static RELATIVE_POS
            centroidAbove ( am::Cuboid          const& above
                            , am::Cuboid        const& below
                            , Eigen::Vector3f   const& up
                            , float                   * angle = NULL );

            static int
            filterContacts( am::Contact::Vec                      & filtered_contacts
                            , am::Contact::Vec               const& contacts
                            , am::Cuboids                    const& cuboids
                            , std::map<int,am::Cuboid::TYPE> const& walls );
    };
}

#endif // __CA_CONTACTSFILTERING_H__
