#ifndef MYINTRINSICS_H
#define MYINTRINSICS_H

//#include "calibration_cuda_constants_prism4.h"
#include <eigen3/Eigen/Dense>

struct MyIntrinsics
{
        float fx, fy, cx, cy;
        float k1, k2, p1, p2, k3, alpha;
        float dc1, dc2;

        MyIntrinsics()
            : fx(0.f), fy(0.f), cx(0.f), cy(0.f), k1(0.f), k2(0.f), p1(0.f), p2(0.f), k3(0.f), alpha(0.f), dc1(0.f), dc2(0.f)
        {}

        MyIntrinsics( float fx, float fy, float cx, float cy,
                      float k1 = 0.f, float k2 = 0.f, float p1 = 0.f, float p2 = 0.f, float k3 = 0.f,
                      float alpha = 0.f, float dc1 = 0.f, float dc2 = 0.f )
            : fx(fx), fy(fy), cx(cx), cy(cy), k1(k1), k2(k2), p1(p1), p2(p2), k3(k3), alpha(alpha), dc1(dc1), dc2(dc2)
        {}

        Eigen::Vector2f f()
        {
            return (Eigen::Vector2f) { fx, fy };
        }
        Eigen::Vector2f c()
        {
            return (Eigen::Vector2f){ cx, cy };
        }

        Eigen::Matrix3f asMatrix3f() const
        {
            Eigen::Matrix3f ret;
            ret <<  fx, alpha,  cx,
                   0.f,    fy,  cy,
                   0.f,   0.f, 1.f;
            return ret;
        }

};

#endif // MYINTRINSICS_H
