#ifndef GMM_H
#define GMM_H

#include <opencv2/imgproc/imgproc.hpp>
#include <memory>
#include <iostream>
#include <eigen3/Eigen/Dense>

#include "IGMM.h"

/*
This is implementation of image segmentation algorithm GrabCut described in
"GrabCut — Interactive Foreground Extraction using Iterated Graph Cuts".
Carsten Rother, Vladimir Kolmogorov, Andrew Blake.
 */

/*
 GMM - Gaussian Mixture Model
*/
template <int cn>
class GMM : public IGMM
{
    public:
        static const     int    kComponentsCount    = 5;
        static constexpr double kVarianceCorrection = .01;

        GMM( cv::Mat& model );
        double operator()( const cv::Vec<double,cn> color ) const;
        double operator()( int ci, cv::Vec<double,cn> color ) const;
        int whichComponent( const cv::Vec<double,cn> color ) const;

        void initLearning();
        void addSample( int ci, const cv::Vec<double,cn> color );
        void endLearning();

    private:
        void calcInverseCovAndDeterm( int ci );
        cv::Mat model_;
        double* p_coefs_;
        double* p_means_;
        double* p_cov_;

        double  inverse_covs_   [kComponentsCount][cn][cn];
        double  cov_determs_    [kComponentsCount];

        double  sums_           [kComponentsCount][cn];
        double  prods_          [kComponentsCount][cn][cn];
        int     sampleCounts_   [kComponentsCount];
        int     totalSampleCount_;

        const int channels_;
};

#endif // GMM_H
