#ifndef GRABCUT_H
#define GRABCUT_H

#include <opencv2/core/core.hpp>
#include <gc_types.h>
#include "IGMM.h"
#include "gcgraph.hpp"

namespace am
{
    namespace my_grabcut
    {
        enum { IMG_RGB = 0, IMG_DEP = 1 };
        enum { GMM_RGB_BG = 0, GMM_RGB_FG = 1, GMM_DEP_BG = 2, GMM_DEP_FG = 3 };
        enum { WEIGHTS_LEFT = 0, WEIGHTS_UPLEFT = 1, WEIGHTS_UP = 2, WEIGHTS_UPRIGHT = 3 };
    } // ns my_grabcut

    class MyGrabCut
    {
        public:
            template <int cn, typename t_img>
            static void initGMMs( const cv::Mat& img, const cv::Mat& mask, GMM<cn>& bgdGMM, GMM<cn>& fgdGMM );

            template <int cn, typename t_img>
            static void assignGMMsComponents( const cv::Mat& img, const cv::Mat& mask, const GMM<cn>& bgdGMM, const GMM<cn>& fgdGMM, cv::Mat& compIdxs );
            template <int cn, typename t_img>
            static void assignGMMsComponents( cv::Mat& compIdxs, const cv::Mat& img, const cv::Mat& mask, const GMM<cn>& bgdGMM, const GMM<cn>& fgdGMM );

            template <int cn, typename t_img>
            static void learnGMMs( const cv::Mat& img, const cv::Mat& mask, const cv::Mat& compIdxs, GMM<cn>& bgdGMM, GMM<cn>& fgdGMM );
            template <int cn, typename t_img>
            static void learnGMMs( GMM<cn>& bgdGMM, GMM<cn>& fgdGMM, const cv::Mat& img, const cv::Mat& mask, const cv::Mat& compIdxs );

            template <int cn, typename t_img>
            static void grabCut( cv::InputArray _img, cv::InputOutputArray _mask, cv::Rect rect,
                                 cv::InputOutputArray _bgdModel, cv::InputOutputArray _fgdModel,
                                 int iterCount, int mode );


    };

    class MyGrabCutD
    {
        public:

            struct params_struct
            {
                    double beta_coeffs[2];
                    params_struct()
                        : beta_coeffs{1.,1.}
                    {}
            };

            template <typename t_img, typename t_dep>
            static void calcNWeights( std::vector<cv::Mat> const& images,
                                      cv::Mat& leftW, cv::Mat& upleftW, cv::Mat& upW, cv::Mat& uprightW,
                                      double *betas, double gamma );

            template <typename t_img, typename t_dep>
            static void constructGCGraph( const std::vector<cv::Mat> &images,
                                          const cv::Mat              &mask,
                                          const std::vector<IGMM*>   &gmms,
                                          double                     lambda,
                                          const std::vector< std::vector< cv::Mat > > &weights,
                                          GCGraph<double> &graph,
                                          params_struct *params = nullptr);

            template <typename t_img>
            static void constructGCGraph4( const std::vector<cv::Mat> &images,
                                          const cv::Mat              &mask,
                                          const std::vector<IGMM*>   &gmms,
                                          double                     lambda,
                                          const std::vector< std::vector< cv::Mat > > &weights,
                                          GCGraph<double> &graph,
                                          params_struct *params = nullptr);

            template <typename t_img, typename t_dep>
            static void
            grabCutD( cv::InputArray       const& images_arg,
                      cv::InputOutputArray const& mask_arg,
                      cv::Rect                    rect,
                      cv::InputOutputArray        &models_arg,
                      int iterCount, int mode,
                      params_struct *params = nullptr );

            template <typename t_img, typename t_dep>
            static void
            grabCutD4( cv::InputArray       const& images_arg,
                       cv::InputOutputArray const& mask_arg,
                       cv::Rect                    rect,
                       cv::InputOutputArray        &models_arg,
                       int iterCount, int mode,
                       params_struct *params );
    };
}

#endif // GRABCUT_H
