#ifndef __CA_CUBOID_HPP__
#define __CA_CUBOID_HPP__

#include "Cuboid.h"

#include "cuboid_approx/util/smartgeometry/pcltools/util.hpp"

#if 1
template <typename CloudPtrT> int
am::Cuboid::fromCloudWithPCA2( Cuboid                              &cuboid_out
                               , CloudPtrT                          cloud
                               , pcl::PointIndices::ConstPtr        selection
                               , pcl::PointIndices::ConstPtr        indices_arg
                               , float                       const  offset )
{
    typedef typename CloudPtrT::element_type CloudT;

    am::Cuboid cuboid;
    pcl::PointIndices::ConstPtr indices = selection ? selection : (indices_arg ? indices_arg : am::util::pcl::allIndicesOf( cloud ));

    // PCA
    {
        Eigen::Matrix4f frame;
        am::DepthProcessor::PCA( frame
                                 , cloud
                                 , indices );
        am::Cuboid::fromFrame( cuboid
                               , frame
                               , cloud
                               , indices );
    }

    std::cout << "[" << __func__ << "]: " << "showing input indices..." << std::endl; fflush( stdout );
//    boost::shared_ptr<am::CloudViewer> vp = am::CloudViewer::showCloud( cloud, "pca2", cuboid.IndicesPtr(), 2., false );
//    vp->ViewerPtr()->setBackgroundColor( .2, .2, .8 );
//    cuboid.draw4( vp->ViewerPtr(), "cuboid" );
//    vp->ViewerPtr()->spin();

    pcl::PointIndices::ConstPtr full_indices = indices_arg ? indices_arg : am::util::pcl::allIndicesOf(cloud);
    cuboid.IndicesPtrRef()->indices = full_indices->indices;
    Cuboid::clampPointsToOrientedPlanes( cuboid_out, cuboid, cuboid.Planes(), -FLT_EPSILON );
    //vp->ViewerPtr()->removeAllPointClouds();
//    vp->ViewerPtr()->addPointCloud( boost::const_pointer_cast<const CloudT>( cuboid_out.CloudPtr() )
//                                    , cuboid_out.IndicesPtr()
//                                    , 0 );
//    CloudViewer::addPointCloud( vp->ViewerPtr(), cuboid_out.CloudPtr(), "cloud", cuboid_out.IndicesPtr(), 0, 2. );
//    std::cout << "[" << __func__ << "]: " << "showing included indices..." << std::endl; fflush( stdout );
//    vp->ViewerPtr()->spin();

    // PCA
    {

        Eigen::Matrix4f frame;
        am::DepthProcessor::PCA( frame
                                 , cuboid_out.CloudPtr()
                                 , cuboid_out.IndicesPtr() );

        am::Cuboid::fromFrame( cuboid_out
                               , frame
                               , cuboid_out.CloudPtr()
                               , cuboid_out.IndicesPtr() );
    }

    return EXIT_SUCCESS;
}
#endif

template <typename PointT> int
am::Cuboid::fromCloudWithPCA( am::Cuboid &cuboid_out
                              , boost::shared_ptr<pcl::PointCloud<PointT> > cloud
                              , boost::shared_ptr<pcl::PointIndices > indices_arg
                              , int   const K
                              , float const radius
                              , int   const filter_nmeans
                              , float const filter_stddev
                              , float const density_cap_median_multiplier
                              , bool  const debug                        )
{
    using std::vector;

    // cleanup
    //typename pcl::PointCloud<PointT>::Ptr cloud( new typename pcl::PointCloud<PointT>() );
    if ( debug && indices_arg ) std::cout << "in_indices: " << indices_arg->indices.size() << std::endl;
    pcl::PointIndices::Ptr indices( new pcl::PointIndices() );
    am::DepthProcessor::statistical_clean<PointT>( indices
                                                   , cloud
                                                   , indices_arg ? &indices_arg : nullptr
                                                   , nullptr
                                                   , filter_nmeans
                                                   , filter_stddev );
    if ( debug && indices_arg ) std::cout << "out_indices: " << indices->indices.size() << std::endl;

    // debug
    boost::shared_ptr<am::CloudViewer> vptr;
    if ( debug ) vptr = am::CloudViewer::showCloud( cloud, "cloud", indices, 2., true );

    /// mean radius
    pcl::PointIndices::Ptr roi_indices( new pcl::PointIndices() );
    vector<vector<int>   > neighbour_indices;
    vector<float         > densities;
    std::pair<float,float> min_max_density( FLT_MAX, 0.f );
    float                  avg_density = 0.f;
    {
        // neighbours
        vector<vector<float> > neighbour_dists;
        smartgeometry::getNeighbourhoodIndices( /* neighbours: */ neighbour_indices
                                                , /*    cloud: */ cloud
                                                , /*  indices: */ &(indices->indices)
                                                , /*    dists: */ &neighbour_dists
                                                , /*   max_ns: */ K
                                                , /*   radius: */ radius );
        // calc densitites
        densities.resize( neighbour_indices.size(), 0.f );
        for ( size_t pid = 0; pid != neighbour_indices.size(); ++pid )
        {
            for ( size_t nid = 0; nid != neighbour_indices[pid].size(); ++nid )
                densities[pid] += sqrt( neighbour_dists[pid][nid] );
            densities[pid] /= static_cast<float>(neighbour_indices[pid].size()) / 1000.f;

            // min, max, avg
            if ( densities[pid] < min_max_density.first  ) min_max_density.first  = densities[pid];
            if ( densities[pid] > min_max_density.second ) min_max_density.second = densities[pid];
            avg_density += densities[pid];
        }
        // avg
        avg_density /= neighbour_indices.size();

        if ( debug )
        {
            std::string   fname = "densities.txt";
            std::ofstream f( fname );

            // colour output
            //const float span = min_max_density.second - min_max_density.first;
            for ( size_t iid = 0; iid != neighbour_indices.size(); ++iid )
            {
                // float ratio = (densities[iid] - min_max_density.first) / span;
                am::util::pcl::setPointColor( cloud->at( indices->indices[iid] )
                                              , std::max(0.f,(10.f - densities[iid]))*255.f, 0.f, 0.f );
                f << cloud->at(indices->indices[iid]).z << " " << densities[iid] << "\n";
            }
            f.close();
            // std::cout << "calling " << ("gnuplot -e \"plot '" + fname + "' u 1:2 w points\" -p") << std::endl;
            // system( ("gnuplot -e \"plot '" + fname + "' u 1:2 w points\" -p").c_str() );
        }
    }

    if ( debug )
    {
        vptr->VP()->setCameraPosition( 0,-4,2, 0,0,3, 0,-1,0 );
        std::cerr << "avg: " << avg_density << std::endl;
        std::cerr << "median: " << am::util::algo::median( densities ) << std::endl;
    }

    am::Cuboid cuboid;
    float cap = am::util::algo::median( densities ) * density_cap_median_multiplier;
    if ( debug ) std::cout << "cap: " << cap << std::endl;

    {
        roi_indices->indices.clear();

        for ( size_t iid = 0; iid != neighbour_indices.size(); ++iid )
        {
            if ( densities[iid] < cap )
            {
                roi_indices->indices.push_back( indices->indices[iid] );

                if ( debug )
                    am::util::pcl::setPointColor( cloud->at(indices->indices[iid])
                                                  , std::max(0.f,(cap - densities[iid]))*255.f, 0.f, 0.f );
            }
        }
        if ( debug ) std::cout << "roi_indices->size: " << roi_indices->indices.size() << std::endl;

        // PCA
        {
            Eigen::Matrix4f frame;
            am::DepthProcessor::PCA( frame
                                     , cloud
                                     , roi_indices );
            am::Cuboid::fromFrame( cuboid
                                   , frame
                                   , cloud
                                   , roi_indices );

            if ( debug )
            {
                vptr->VP()->removeAllShapes();
                cuboid.draw( vptr->VP()
                             , "cuboid"
                             , (Eigen::Vector3f() << .9, .0, .9).finished()
                             , 5.
                             , true );

            }
        }

        if ( debug )
        {
            am::CloudViewer::updatePointCloud( vptr->VP(), cloud, "cloud", indices );
            vptr->VP()->setPointCloudRenderingProperties( pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 2.5, "cloud", 0 );
            vptr->VP()->spin();
        }
    }

    //cuboid.IndicesPtrRef()->indices = am::util::pcl::allIndicesOf( cloud )->indices;
    cuboid.IndicesPtrRef()->indices = indices->indices;
    //am::Cuboid cuboid2;
    Cuboid::clampPointsToOrientedPlanes( cuboid_out, cuboid, cuboid.Planes(), -FLT_EPSILON );

    if ( debug )
    {
        for ( size_t pid = 0; pid != cuboid.IndicesPtr()->indices.size(); ++pid )
            am::util::pcl::setPointColor( cloud->at(cuboid.IndicesPtr()->indices[pid]), 0.f, 255.f, 255.f );
        for ( size_t pid = 0; pid != cuboid_out.IndicesPtr()->indices.size(); ++pid )
            am::util::pcl::setPointColor( cloud->at(cuboid_out.IndicesPtr()->indices[pid]), 255.f, 255.f, 0.f );
    }

    // PCA
    {
        Eigen::Matrix4f frame;
        am::DepthProcessor::PCA( frame
                                 , cuboid_out.CloudPtr()
                                 , cuboid_out.IndicesPtr() );

        am::Cuboid::fromFrame( cuboid_out
                               , frame
                               , cuboid_out.CloudPtr()
                               , cuboid_out.IndicesPtr() );

        if ( debug )
        {
            vptr->VP()->removeAllShapes();
            cuboid_out.draw( vptr->VP(), "cuboid2" );
        }
    }

    if ( debug )
    {
        am::CloudViewer::updatePointCloud( vptr->VP(), cloud, "cloud", indices );
        vptr->VP()->setPointCloudRenderingProperties( pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 2.5, "cloud", 0 );
        vptr->VP()->spin();
    }

    return EXIT_SUCCESS;
}

#endif // __CA_CUBOID_HPP__
