#ifndef CUBOIDS_H
#define CUBOIDS_H

#include "PrimitiveContainer.h"
#include "Cuboid.h"

namespace am
{
    class GrabcutResult;
    class Contact;

    class Cuboids : public PrimitiveContainer<Cuboid>
    {
        public:
            template <typename depT>
            static int
            from_masks( Cuboids             &cuboids, GrabcutResult const &masks,
                        PCloud::ConstPtr    cloud_ptr,
                        pcl::PointIndices::ConstPtr indices_ptr
                        , double distanceThreshold = .2 );

            static int
            writeToFile(Cuboids const& cuboids, std::string path , int height, int width);
            static int
            writeToTextFile(Cuboids const& cuboids, std::string path , int height, int width);

            static int
            writeLabels( Cuboids const& cuboids, std::string path, int height, int width );

            static int
            readFromFile( Cuboids                                 &cuboids
                          , std::string                     const &path
                          , std::string                     const *inliers_path           = NULL
                          , PCloud::ConstPtr                       cloud_ptr              = nullptr
                          , pcl::PointIndices::ConstPtr            indices_ptr            = nullptr
                          , std::vector<am::Contact>              *contacts               = NULL
                          , std::vector<am::Primitive::Ptr>       *statics                = NULL
                          , double                          const *plane_inlier_threshold = NULL
                          , bool                                   verbose                = false
                          );

            static std::string
            getLabelsPath( std::string path );


            static int
            testReadWrite();
    };

} // ns am

#ifndef INC_CUBOIDS_HPP
#   define INC_CUBOIDS_HPP
#   include "Cuboids.hpp"
#endif

#endif // CUBOIDS_H
