#ifndef CUBOIDS_HPP
#define CUBOIDS_HPP

#include "DepthProcessor.h"

#include "Cuboids.h"
#include "GrabcutResult.h"

namespace am
{
    template <typename depT>
    int
    Cuboids::from_masks( Cuboids                     &cuboids,
                         GrabcutResult         const &masks,
                         PCloud::ConstPtr            cloud_ptr,
                         pcl::PointIndices::ConstPtr indices_ptr
                         , double distanceThreshold )
    {
        std::cout << "DepthProcessor::cuboids_from_masks starting..." << std::endl;

        int err = EXIT_SUCCESS;

        cuboids.clear();

        for ( int label = 0; label < 1000; ++label )
        {
            // debug
            std::cout << "label " << label << std::endl;

            cv::Mat masked_depth;
            {
                masks.getBinaryMaskForLabel( masked_depth, label );
            }

            // no more masks?
            if ( masked_depth.empty() )
            {
                std::cout << "masked_depth.empty(), exiting" << std::endl;
                break;
            }

            // 3d point indices
            pcl::PointIndices::Ptr masked_indices_ptr;
            {
                DepthProcessor::getMaskedIndices<depT>( masked_indices_ptr, masked_depth, masks.get_depth() );
                if ( !masked_indices_ptr->indices.size() )
                {
                    std::cout << "!masked_indices_ptr->indices.size()" << std::endl;
                    break;
                }
            }

            // apply selection
            if ( indices_ptr && indices_ptr->indices.size() )
            {
                am::util::algo::vectorIntersection( /* out: */ masked_indices_ptr->indices,
                                                    /* in1: */ masked_indices_ptr->indices,
                                                    /* in2: */ indices_ptr->indices );
            }

            // extract cuboid
            Cuboid cuboid;
            {
                err = Cuboid::from_cloud( cuboid,
                                          cloud_ptr,
                                          masked_indices_ptr,
                                          distanceThreshold
                                          , false );
                if ( err != EXIT_SUCCESS )
                {
                    std::cerr << "DepthProcessor::cuboid_from_cloud() exited with " << (int)err << "\n";
                    return err;
                }
            }

            // save cuboid
            {
                cuboids.add( cuboid );
            }
        }

        std::cout << "DepthProcessor::cuboids_from_masks finished..." << std::endl;

        return err;
    }
}

#endif // CUBOIDS_HPP
