#ifndef PLANES_H
#define PLANES_H

#include "opencv2/core/core.hpp"
#include "PrimitiveContainer.h"
#include "Plane.h"

namespace am
{
    class Cuboid;
    class Params;
    class GrabcutResult;

    class Planes : public PrimitiveContainer<Plane>
    {
        public:
            static int
            from_cuboid2( am::Planes        &planes,
                          am::Cuboid const  &cuboid,
                          am::Params const  &params,
                          double distanceThreshold = .1 );
            static int
            from_cuboid( am::Planes        &planes,
                         am::Cuboid const  &cuboid,
                         am::Params const  &params,
                         double distanceThreshold = .1,
                         bool debug = false );

            static int
            from_cloud( am::Planes                          &planes
                        , PCloud::ConstPtr                  cloud_ptr
                        , pcl::PointIndices::ConstPtr       indices_ptr
                        , am::Params                  const &params
                        , double                            distanceThreshold = .1
                        , bool                              debug = false );

            template <typename depT>
            static int
            from_masks( Planes                            &planes,
                        GrabcutResult               const &masks,
                        PCloud::ConstPtr                   cloud_ptr,
                        pcl::PointIndices::ConstPtr        indices_ptr,
                        double                             distanceThreshold );

            int
            colourCloud( PCloud::Ptr cloud_ptr,
                         std::vector<cv::Point3f> const* arg_p_colors = nullptr ) const;

            static int
            writeToFile( Planes const& planes, std::string path );
            static int
            readFromFile( Planes &planes, std::string const& path );
            static int
            testReadWrite();
    };

} // ns am

#ifndef INC_PLANES_HPP
#   define INC_PLANES_HPP
#   include "Planes.hpp"
#endif // INC_PLANES_HPP

#endif // PLANES_H
