#ifndef PLANES_HPP
#define PLANES_HPP

#include "Planes.h"
#include "GrabcutResult.h"

#include "DepthProcessor.h"

namespace am
{
    template <typename depT>
    int
    Planes::from_masks( Planes                             &planes,
                        GrabcutResult                const &masks,
                        PCloud::ConstPtr                    cloud_ptr,
                        pcl::PointIndices::ConstPtr         indices_ptr,
                        double                              distanceThreshold )
    {
        int err = EXIT_SUCCESS;

        // prepare output
        planes.clear();

        for ( int label = 0; label < 1000; ++label )
        {
            cv::Mat masked_depth;
            masks.getBinaryMaskForLabel( masked_depth, label );

            // no more masks?
            if ( masked_depth.empty() )
                break;

            // 3d point indices
            pcl::PointIndices::Ptr masked_indices_ptr;
            am::DepthProcessor::getMaskedIndices<depT>( masked_indices_ptr,
                                                        masked_depth,
                                                        masks.get_depth() );

            // apply selection
            if ( indices_ptr && indices_ptr->indices.size() )
            {
                am::util::algo::vectorIntersection( /* out: */ masked_indices_ptr->indices,
                                                    /* in1: */ masked_indices_ptr->indices,
                                                    /* in2: */ indices_ptr->indices );
            }

            // extract cuboid
            Plane plane;
            {
                err = Plane::fromCloud( plane,
                                        cloud_ptr,
                                        masked_indices_ptr,
                                        distanceThreshold );
                if ( err != EXIT_SUCCESS )
                {
                    std::cerr << "DepthProcessor::cuboid_from_cloud() exited with " << (int)err << "\n";
                    return err;
                }
            }

            // save cuboid
            {
                planes.add( plane );
            }
        }

        return err;

    }

}

#endif // PLANES_HPP
