#ifndef PRIMITIVE_H
#define PRIMITIVE_H

#include "pcl/point_cloud.h"
#include "pcl/PointIndices.h"
#include "pcl/visualization/pcl_visualizer.h"

#include "cuboid_approx/common.h"

namespace am
{
    class Primitive
    {
        public:
            typedef std::shared_ptr<      am::Primitive>      Ptr;
            typedef std::shared_ptr<const am::Primitive> ConstPtr;
            //enum Shape { CUBOID, SPHERE };
            /// STATIC METHODS

            /// CONSTR
            Primitive();
            Primitive( PCloud::ConstPtr cloud_ptr );

            /// METHODS
            virtual int
            unDraw( pcl::visualization::PCLVisualizer::Ptr v,
                    std::string                            cloud_name );

            virtual int
            cloneTo( Primitive & ) const;

            // ROI cloud
            int
            cloneROICloud( PCloud::Ptr &cloud_ptr );
            PCloud::Ptr const&
            getROICloud() const;

            /// SET, GET
            // _cloud_ptr
            PCloud::ConstPtr               CloudPtr() const;
            PCloud::ConstPtr             & CloudPtrRef();
            // _indices_ptr
            pcl::PointIndices::Ptr const   IndicesPtr() const;
            pcl::PointIndices::Ptr       & IndicesPtrRef();
            pcl::PointIndices::Ptr         IndicesNonConstPtr();

        protected:
            PCloud::ConstPtr        _cloud_ptr;      // hosting cloud
            pcl::PointIndices::Ptr  _indices_ptr;    // indices of hosting cloud comprising this primitive
            mutable PCloud::Ptr     _roi_cloud_ptr;  // = _cloud_ptr.select(_indices_ptr)
            mutable bool            _indicesChanged; // dirty flag for roi_cloud_ptr and _indices_ptr

            Primitive operator=( Primitive const& );
    };

} // ns am

#endif // PRIMITIVE_H
