#ifndef CLOUDVIEWER_H
#define CLOUDVIEWER_H

#include <string>
#include "pcl/visualization/pcl_visualizer.h"
#include "cuboid_approx/common.h"

namespace am
{

    class CloudViewer
    {
        public:
            // CNSTR
            CloudViewer( std::string title = "Color Cloud Viewer", int nviewports = 1 );

            static boost::shared_ptr<am::CloudViewer>
            showCloud( PCloud::ConstPtr            cloud_ptr,
                       std::string                 title       = "CloudViewer::showCloud",
                       pcl::PointIndices::ConstPtr indices_ptr = nullptr,
                       double                      point_size  = 2.,
                       bool                        doSpin      = true );

            static boost::shared_ptr<am::CloudViewer>
            showCloud( pcl::PointCloud<pcl::PointXYZRGB>::ConstPtr cloud_ptr,
                       std::string                 title       = "CloudViewer::showCloud",
                       pcl::PointIndices::ConstPtr indices_ptr = nullptr,
                       double                      point_size  = 2.,
                       bool                        doSpin      = true );

            static int
            addCloud( pcl::visualization::PCLVisualizer::Ptr viewer_ptr,
                      PCloud::ConstPtr                       cloud_ptr,
                      std::string                            title       = "cloud",
                      pcl::PointIndices::ConstPtr            indices_ptr = nullptr,
                      int                                    viewport_id = 0,
                      double                                 point_size  = 1.);

            static int
            addPointCloud( pcl::visualization::PCLVisualizer::Ptr viewer_ptr,
                           PCloud::ConstPtr                       cloud_ptr,
                           std::string                            title       = "cloud",
                           pcl::PointIndices::ConstPtr            indices_ptr = nullptr,
                           int                                    viewport_id = 0,
                           double                                 point_size  = 1.);
            static int
            updatePointCloud( pcl::visualization::PCLVisualizer::Ptr viewer_ptr,
                              PCloud::ConstPtr                       cloud_ptr,
                              std::string                            title       = "cloud",
                              pcl::PointIndices::ConstPtr            indices_ptr = nullptr,
                              int                                    viewport_id = 0,
                              double                                 point_size  = 1. );
            static int
            show2DFromLabels( PCloud::ConstPtr              cloud_ptr
                              , int                         width
                              , int                         height
                              , std::string                 title = "cloud"
                              , pcl::PointIndices::ConstPtr indices_ptr = nullptr
                              );

            int
            addCloud( PCloud::ConstPtr            cloud_ptr,
                      std::string                 title       = "cloud",
                      pcl::PointIndices::ConstPtr indices_ptr = nullptr,
                      int                         viewport_id = 0,
                      double                      point_size  = 1.);
            int
            addPointCloud( PCloud::ConstPtr            cloud_ptr,
                           std::string                 title       = "cloud",
                           pcl::PointIndices::ConstPtr indices_ptr = nullptr,
                           int                         viewport_id = 0,
                           double                      point_size  = 1.);


            pcl::visualization::PCLVisualizer::Ptr ViewerPtr() { return viewer_ptr_; }
            pcl::visualization::PCLVisualizer::Ptr getVPtr() { return viewer_ptr_; }
            pcl::visualization::PCLVisualizer::Ptr VP() { return viewer_ptr_; }
            PCloud::Ptr CloudPtr()  { return cloud_ptr_;  }
            int getVid( int vid );
            virtual ~CloudViewer();

        protected:
            // FIELDS
            pcl::visualization::PCLVisualizer::Ptr viewer_ptr_;
            PCloud::Ptr cloud_ptr_;
            std::vector<int> _viewport_ids;

    };

} // ns am

#endif // CLOUDVIEWER_H
