#ifndef __CA_MYVOXELGRID_HPP__
#define __CA_MYVOXELGRID_HPP__

#include "MyVoxelGrid.h"

#include <pcl/point_cloud.h>
#include <pcl/visualization/cloud_viewer.h>
#include <pcl/filters/voxel_grid.h>

#include "cuboid_approx/util/AMUtil2.h"

namespace am
{
    template <typename PointT>
    int
    MyVoxelGrid::drawGrid( pcl::visualization::PCLVisualizer::Ptr        v_ptr
              , std::vector<int>                     const& leafs
              , pcl::VoxelGrid<PointT>                    * sor  )
    {
        const auto divs    = sor->getNrDivisions();
        const auto div_mul = sor->getDivisionMultiplier();
        Eigen::Vector3i ijk;
        for ( ijk(0) = 0; ijk(0) < divs(0); ++ijk(0) )
            for ( ijk(1) = 0; ijk(1) < divs(1); ++ijk(1) )
                for ( ijk(2) = 0; ijk(2) < divs(2); ++ijk(2) )
                {
                    // debug a plane
                    //                        if (!( ijk(0) >= 1
                    //                             && ijk(0) <= 14
                    //                             && ijk(1) == 6
                    //                             && ijk(2) >= 2
                    //                             && ijk(2) <= 6 ) )
                    //                            continue;

                    char str[255];
                    sprintf( str, "(%d,%d,%d)=%d", ijk(0),ijk(1),ijk(2), leafs[ijk.dot(div_mul)] );
                    //vptr->VP()->update

                    std::string id = MyVoxelGrid::getCellGridId( ijk );
                    v_ptr->removeText3D( id );
                    v_ptr->addText3D( /*  text: */   std::string(str)
                                      /*   pos: */ , am::util::pcl::asPointXYZ(getCentroidCoord(ijk,sor))
                                      /* scale: */ , 0.01
                                      /*   red: */ , 0.
                                      /* green: */ , 1.
                                      /*  blue: */ , 0.
                                      /* label: */ , id );
                }

        return EXIT_SUCCESS;
    }

    template <typename PointT>
    int
    MyVoxelGrid::drawRealGrid( pcl::visualization::PCLVisualizer::Ptr        v_ptr
                               , std::vector<int>                     const& leafs
                               , pcl::VoxelGrid<PointT>                    * sor
                               )
    {
        const auto divs    = sor->getNrDivisions();
        const auto div_mul = sor->getDivisionMultiplier();
        const auto min_box_coords = sor->getMinBoxCoordinates();
        const auto leaf_size = sor->getLeafSize();

        Eigen::Vector3i ijk(0,0,0);
        for ( ijk(0) = 0; ijk(0) < divs(0); ++ijk(0) )
            for ( ijk(1) = 0; ijk(1) <= divs(1); ++ijk(1) )
                for ( ijk(2) = 0; ijk(2) <= divs(2); ++ijk(2) )
                {
                    char str[255];

                    std::vector<Eigen::Vector3i> corners = {{0,0,0}, {0,0,1},
                                                            {0,0,1}, {0,1,1},
                                                            {0,1,1}, {0,1,0},
                                                            {0,1,0}, {0,0,0},
                                                            {1,0,0}, {1,0,1},
                                                            {1,0,1}, {1,1,1},
                                                            {1,1,1}, {1,1,0},
                                                            {1,1,0}, {1,0,0},
                                                            {0,0,0}, {1,0,0},
                                                            {0,0,1}, {1,0,1},
                                                            {0,1,1}, {1,1,1},
                                                            {0,1,0}, {1,1,0}
                                                           };
                    for ( int c = 0; c < corners.size(); c += 2 )
                    {
                        pcl::PointXYZ start( (ijk(0)+corners[c  ](0)+min_box_coords(0)) * leaf_size(0),
                                             (ijk(1)+corners[c  ](1)+min_box_coords(1)) * leaf_size(1),
                                             (ijk(2)+corners[c  ](2)+min_box_coords(2)) * leaf_size(2)  );
                        pcl::PointXYZ stop ( (ijk(0)+corners[c+1](0)+min_box_coords(0)) * leaf_size(0),
                                             (ijk(1)+corners[c+1](1)+min_box_coords(1)) * leaf_size(1),
                                             (ijk(2)+corners[c+1](2)+min_box_coords(2)) * leaf_size(2)  );
                        static int lineid = 0;
                        sprintf( str, "line%d", lineid++ );
                        v_ptr->addLine( stop
                                        , start
                                        , 0.f/255.f, 169.f/255.f, 151.f/255.f

                                        , str );
                        v_ptr->setShapeRenderingProperties( pcl::visualization::PCL_VISUALIZER_LINE_WIDTH, 3., str );
                    }
                }
//                    Eigen::Vector3i tmpijk = ijk;
//                    for ( int i = 0; i < 2; ++i )
//                    {
//                        for ( int d = 0; d < 3; ++d )
//                        {
//                            Eigen::Vector3i tmpijk2 = tmpijk;
//                            tmpijk2(d) += (i?-1:1);
//                            std::cout << "line: " << tmpijk.transpose() << " -> " << tmpijk2.transpose() << std::endl;

//                            pcl::PointXYZ start( (tmpijk2(0)+min_box_coords(0)) * leaf_size(0),
//                                                 (tmpijk2(1)+min_box_coords(1)) * leaf_size(1),
//                                                 (tmpijk2(2)+min_box_coords(2)) * leaf_size(2)  );
//                            pcl::PointXYZ stop( ( tmpijk(0)+min_box_coords(0)) * leaf_size(0),
//                                                ( tmpijk(1)+min_box_coords(1)) * leaf_size(1),
//                                                ( tmpijk(2)+min_box_coords(2)) * leaf_size(2)  );
//                            sprintf( str, "%d_%d_%d_%d_%d_%d", tmpijk2(0), tmpijk2(1), tmpijk2(2), tmpijk(0),tmpijk(1),tmpijk(2) );
//                            v_ptr->addLine( stop, start
//                                            , 1.f, 1.f, 0.f
//                                            , str );
//                            v_ptr->setShapeRenderingProperties( pcl::visualization::PCL_VISUALIZER_LINE_WIDTH, 3., str );

//                            v_ptr->addText3D( str, start, .01, 1., 0., 0., std::string(str)+"label" );
//                        }
//                        tmpijk += (Eigen::Vector3i){1,1,1};
//                    }
                //}

        return EXIT_SUCCESS;
    }

    template <typename PointT>
    int
    MyVoxelGrid::drawCentroids( pcl::visualization::PCLVisualizer::Ptr v_ptr
                   , pcl::VoxelGrid<PointT>               *sor
                   , std::string                          title
                   , int                                  viewport
                                                                                                                       )
    {
        const Eigen::Vector3i divs           = sor->getNrDivisions();
        const Eigen::Vector3i min_box_coords = sor->getMinBoxCoordinates();
        const Eigen::Vector3f leaf_size      = sor->getLeafSize();

        boost::shared_ptr<pcl::PointCloud<PointT> > cloud_ptr( new pcl::PointCloud<PointT> );

        for ( int i = 0; i < divs[0]; ++i )
            for ( int j = 0; j < divs[1]; ++j )
                for ( int k = 0; k < divs[2]; ++k )
                {
                    Eigen::Vector3i ijk( i, j, k );
                    // check, if occupied
                    int  id  = sor->getCentroidIndexAt( ijk + min_box_coords );

                    // create point
                    auto pnt = am::util::pcl::PointXYZRGB(
                                   (i+.5f+min_box_coords(0)) * leaf_size(0),
                                   (j+.5f+min_box_coords(1)) * leaf_size(1),
                                   (k+.5f+min_box_coords(2)) * leaf_size(2),
                                   ( id == -1 ) ? 255 : 128,
                                   0,
                                   ( id != -1 ) ? 255 : 0 );
                    // add point
                    cloud_ptr->push_back( pnt );

                }
        return v_ptr->addPointCloud( cloud_ptr, title, viewport );
    }

} // ns am

#endif // __CA_MYVOXELGRID_HPP__
