#ifndef SEGMENTATION_H
#define SEGMENTATION_H

#include "pcl/point_cloud.h"
#include "pcl/point_types.h"
#include "pcl/ModelCoefficients.h"
#include "pcl/PointIndices.h"
#include "pcl/segmentation/sac_segmentation.h"

namespace am
{

    class Segmentation
    {
        public:
            template <typename MyPointT>
            static int
            extract_cylinder( boost::shared_ptr<pcl::PointCloud<MyPointT>>        &cylinder_cloud,
                              pcl::ModelCoefficients::Ptr                         &cylinder_coefficients,
                              boost::shared_ptr<const pcl::PointCloud<MyPointT>>  in_cloud               );

            template <typename MyPointT>
            static int
            extract_plane( pcl::ModelCoefficients::Ptr                         &out_coefficients,
                           boost::shared_ptr<const pcl::PointCloud<MyPointT>>  in_cloud,
                           pcl::PointIndices::ConstPtr                         in_indices        = nullptr,
                           double                                              distanceThreshold = .1,
                           int                                                 sacmodel          = pcl::SACMODEL_NORMAL_PLANE,
                           Eigen::Vector3f                                     *p_normal         = nullptr );

            template <typename MyPointT>
            static int
            region_grow( std::vector <pcl::PointIndices::Ptr>                &cluster_ptrs,
                         boost::shared_ptr<const pcl::PointCloud<MyPointT> > in_cloud_ptr,
                         pcl::PointIndices::ConstPtr                         in_indices_ptr,
                         int                                                 normalK = 50 );

        protected:
            // resize normals cloud to match cloud_ptr, copy estimated ones to right positions, leave others as 0
            static int
            _expandNormals( pcl::PointCloud<pcl::Normal>::Ptr      &cloud_normals_ptr,
                            pcl::PointIndices::ConstPtr             in_indices_ptr,
                            pcl::PointCloud<pcl::Normal>::ConstPtr  roi_cloud_normals_ptr,
                            int output_size );
    };

} // ns am

#ifndef INC_SEGMENTATION_HPP
#   define INC_SEGMENTATION_HPP
#   include "Segmentation.hpp"
#endif // INC_SEGMENTATION_HPP

#endif // SEGMENTATION_H
