#ifndef QTCLOUDVIEWER_H
#define QTCLOUDVIEWER_H

#include <memory>
#include <QMainWindow>
#include <boost/property_tree/ptree.hpp>

#include "AnnotationController.h"
#include "RotatorThread.h"

namespace Ui
{
    class QtCloudViewer;
}

class QtCloudViewer : public QMainWindow
{
    Q_OBJECT

    public:
        explicit QtCloudViewer(QWidget *parent = 0);
        ~QtCloudViewer();

        void updateQvtWidget();

        inline int
        setController( std::shared_ptr<am::AnnotationController> controller_ptr );
        inline std::shared_ptr<am::AnnotationController>
        getController();


//        int
//        saveGTContacts( std::vector<am::Contact> const& contacts, std::string const& str_gt_path );
        
    public Q_SLOTS:
        void onProgressChanged                      ( QString       info        );
        int  loadWalls                              ( std::string   path        );
        void my_on_loadButton_clicked               ( std::string   path = ""   );
        void my_on_addPointsButton_clicked          ( std::string   path        );
        void my_on_loadContactsButton_clicked       ( std::string   path = ""   );
        void my_on_loadCameraParamsButton_clicked   ( std::string   path = ""   );
        void my_on_screenshotButton_clicked         ( std::string   path = ""   );
        void on_sphereSizeSlider_valueChanged       ( int           value       );
        int  centerCamera();
        void on_loadWallsButton_clicked();
    private Q_SLOTS:

        void on_annotateButton_clicked();
        void on_imagePathButton_clicked();
        void on_depthPathButton_clicked();

        void on_clearButton_clicked();

        void on_planeInlierThresholdSlider_sliderMoved(int position);
        void on_planeInlierThresholdDoubleSpinBox_valueChanged(double arg1);

        void on_saveButton_clicked();

        void on_removePrimitiveButton_clicked();

        void on_primitivesListWidget_currentRowChanged(int currentRow);

        void on_pointSizeSpinBox_valueChanged(double arg1);

        void on_resetCameraButton_clicked();

        void on_hidePrimitiveButton_clicked();

        void on_fillButton_clicked();

        void on_showInvalidCloud_toggled(bool checked);

        void on_isFloorCheckBox_clicked(bool checked);
        void on_isWallCheckBox_clicked(bool checked);
        void on_isSphereCheckBox_clicked(bool checked);
        void on_isCylinderCheckBox_clicked(bool checked);

        void on_segmentButton_clicked();

        void on_primitivesListWidget_itemSelectionChanged();

        void on_infoButton_clicked();

        void on_xMoveSlider_sliderMoved(int position);
        void on_yMoveSlider_sliderMoved(int position);
        void on_zMoveSlider_sliderMoved(int position);

        void on_xResizeSlider_sliderMoved(int position);
        void on_yResizeSlider_sliderMoved(int position);
        void on_zResizeSlider_sliderMoved(int position);

        void on_xRotateSlider_sliderMoved(int position);
        void on_yRotateSlider_sliderMoved(int position);
        void on_zRotateSlider_sliderMoved(int position);

        void _sliderReleasedSlot();

        void on_printButton_clicked();

        void on_clusterButton_clicked();

        void on_spatialSigmaSlider_valueChanged(int value);
        void on_spatialSigmaSpinBox_valueChanged(double value);
        void on_rangeSigmaSlider_valueChanged(int value);
        void on_rangeSigmaSpinBox_valueChanged(double value);
        void on_kernelRadiusSlider_valueChanged(int value);
        void on_kernelRadiusSpinBox_valueChanged(int value);

        void on_iterationsSlider_valueChanged(int value);
        void on_iterationsSpinBox_valueChanged(int value);
        void _filteringParamChanged( int id );

        void on_colourCuboids_stateChanged(int arg1);

        void on_colorButton_clicked();

        void on_opacitySlider_valueChanged( int value );

        void on_opacitySpinBox_valueChanged( double value );

        void on_cameraInfoButton_clicked();

        void on_setCameraParamsButton_clicked();

        void on_showIntersections_toggled(bool checked);



        int _addPoints( std::string path );
        void my_on_addPointsButton_clicked();

        void on_removePointsButton_clicked();
        void on_saveCameraParamsButton_clicked();

        void on_sphereSizeSpinBox_editingFinished();

        void on_recordRowButton_clicked();

        void on_colourEdgesCheckBox_stateChanged(int arg1);

        void on_addModelButton_clicked();

        void on_showIntersections_clicked(bool checked);

        void on_saveWallsButton_clicked();

        void on_rotateCameraCheckBox_clicked();

        void on_isTargetCheckBox_clicked(bool checked);

        void on_unselectButton_clicked();

        void on_changeAnimationSavePathButton_clicked();

        void on_getRotInfoButto_clicked();

        void on_loadRegionsButton_clicked();

        void on_loadMasksButton_clicked();

        void on_regGrowButton_clicked();

        void on_createCuboidButton_clicked();

        void on_clearPickButton_clicked();

        void on_save2PLYButton_clicked();

        void on_clearPrimitivesButton_clicked();

        void on_filterContactsButton_clicked();

        void on_colourCloudCheckBox_toggled(bool checked);

        void on_showIDsCheckBox_stateChanged(int arg1);

        void on_moosPlanesButton_clicked();

        void on_savePointsButton_clicked();

        void on_wshopDumpButton_clicked();

        void on_duplicateCuboidButton_clicked();

    protected:
        void                        _refreshPrimitivesList      ();
        void                        _setDepthPath               ( QString fileName );
        std::vector<int>            _getSelectedRows            ();
        void                        _movePrimitive              ( char axis, float step );
        void                        _resizePrimitive            ( char axis, float step );
        void                        _rotatePrimitive            ( char axis, float step );
        int                         _refreshDisplayProperties   ();
        void                        _set_camera_params          ( std::string params_str );
        int                         _saveWalls                  ( std::string path );
        int                         _saveLastPath               ( std::string str );
        std::string                 _getLastPath                ( std::string default_value = "");
        std::pair<int,std::string>  _rotateCamera               ( bool rotate );
        QString                     _getDepthDir                ();
        std::string                 _getDepthDirAsStdString     ();
        std::string                 _getFrameIdFromDepthPath    ();
        void                        _clearPrimitives            ();
        int                         _setContacts                ( std::vector<am::Contact> const& contacts );
        int                         _extractWallsMap            ( std::map<int,am::Cuboid::TYPE> &walls );

        Ui::QtCloudViewer                          *_ui;
        std::shared_ptr<am::AnnotationController>   _controller;
        boost::property_tree::ptree                 _pt;


        RotatorThread                              *_thread;
        std::string                                 _animation_save_path;
        int                                         _frame_count;

        std::string                                 _last_cuboids_path;
        std::string                                 _last_contacts_path;
};

inline int
QtCloudViewer::setController( std::shared_ptr<am::AnnotationController> controller_ptr )
{
    _controller = controller_ptr;
    return EXIT_SUCCESS;
}

inline std::shared_ptr<am::AnnotationController>
QtCloudViewer::getController()
{
    return _controller;
}

#endif // QTCLOUDVIEWER_H
