#include "CuboidUtil.h"

#include "cuboid_approx/util/AMUtil2.h"
#include "opencv2/highgui/highgui.hpp"

namespace am
{
    cv::Mat
    CuboidUtil::loadDepth( std::string depth_path )
    {
        cv::Mat dep;
        if ( depth_path.find("raw") != std::string::npos )
        {
            am::util::cv::loadDepthAsBinary<ushort,ushort>( dep, depth_path, 1. );
        }
        else if ( depth_path.find("dat") != std::string::npos )
        {
            // Tianjia's NYU, read float, and convert it to ushort
            am::util::cv::loadDepthAsBinary<ushort,float>( dep, depth_path, 1000. );
            {
                double minVal, maxVal;
                cv::minMaxLoc( dep, &minVal, &maxVal );
                //std::cout << "minmaxnyu: " << minVal << ", " << maxVal << std::endl;
            }
        }
        else
        {
            dep = cv::imread( depth_path, cv::IMREAD_UNCHANGED );
        }

        return dep;
    }

#if 0
    // returns rowmajor eigen_vectors in decreasing order
    int
    DepthProcessor::PCA( Eigen::Matrix4f             &frame,
                         PCloud::ConstPtr             cloud_ptr,
                         pcl::PointIndices::ConstPtr  indices_ptr )

    {
            Eigen::Vector4f centroid;
            Eigen::Matrix3f covariance;
            if ( indices_ptr )
            {
                pcl::compute3DCentroid( /* cloud: */ *cloud_ptr, *indices_ptr, centroid );
                pcl::computeCovarianceMatrix( *cloud_ptr, *indices_ptr, centroid, covariance );
            }
            else
            {
                pcl::compute3DCentroid( /* cloud: */ *cloud_ptr, centroid );
                pcl::computeCovarianceMatrix( *cloud_ptr, centroid, covariance );
            }

            // eigen
            Eigen::SelfAdjointEigenSolver<Eigen::Matrix3f> eigen_solver( covariance, Eigen::ComputeEigenvectors );
            Eigen::Matrix3f eigen_vectors = eigen_solver.eigenvectors();
            am::util::eigen::sort_eigen_vectors_desc( eigen_solver.eigenvalues(),
                                                      eigen_vectors );
            eigen_vectors.col(2) = eigen_vectors.col(0).cross( eigen_vectors.col(1) );

            // debug
            if ( eigen_vectors(0,0) != eigen_vectors(0,0)
                 || eigen_vectors(1,1) != eigen_vectors(1,1)
                 || eigen_vectors(2,2) != eigen_vectors(2,2) )
            {
                std::cerr << "nan eigen matrix" << std::endl;
                return EXIT_FAILURE;
            }

            frame = Eigen::Matrix4f::Identity();
            frame.block<3,3>(0,0) = eigen_vectors;
            frame.block<3,1>(0,3) = centroid.head<3>();
    }

    cuboid.CenterRef() = vertex_list.rowwise().mean();
    Eigen::MatrixXf vertices = vertex_list.colwise() - cuboid.Center();

    // covariance
    Eigen::MatrixXf cov = vertices * vertices.adjoint();

    // eigen solve
    Eigen::SelfAdjointEigenSolver<Eigen::MatrixXf> eigen_solver( cov );

    Eigen::Matrix3f    eigen_vectors = eigen_solver.eigenvectors().rightCols(3);
    std::vector<float> eigen_values  = { eigen_solver.eigenvalues()(0),
                                         eigen_solver.eigenvalues()(1),
                                         eigen_solver.eigenvalues()(2) };

    std::vector<int> eigen_vector_ids = { 0, 1, 2 };
    am::util::algo::sort3_desc( eigen_values[0], eigen_values[1], eigen_values[2], &eigen_vector_ids );

    cuboid.RRef().col(0) = eigen_vectors.col( eigen_vector_ids[0] );
    cuboid.RRef().col(1) = eigen_vectors.col( eigen_vector_ids[1] );
    cuboid.RRef().col(2) = eigen_vectors.col( eigen_vector_ids[2] );

    cuboid.RRef().col(2) = cuboid.RRef().col(0).cross( cuboid.RRef().col(1) );
#endif

} // ns am
