#include "Params.h"

#include <stdexcept>
#include <iostream>

namespace am
{
    IntParam* Param::asIntParam()
    {
        return reinterpret_cast<IntParam*>( this );
    }

    DoubleParam* Param::asDoubleParam()
    {
        return reinterpret_cast<DoubleParam*>( this );
    }

    StringParam* Param::asStringParam()
    {
        return reinterpret_cast<StringParam*>( this );
    }

    IntParam::IntParam( int value )
        : _value(value)
    {
    }

    DoubleParam::DoubleParam( double value )
        : _value(value)
    {
    }

    StringParam::StringParam( std::string value )
        : _value( value )
    {
    }

    int
    Params::setInt( std::string key, int val )
    {
        _params[key] = new IntParam( val );
        return EXIT_SUCCESS;
    }

    int
    Params::getInt( std::string key, int default_value ) const
    {
        if ( this->hasKey(key) )
        {
            return _params.at( key )->asIntParam()->_value;
        }
        else
        {
            std::cerr << "Params::getInt( " << key << ") empty, returning default value " << default_value << std::endl;
            return default_value;
        }
    }

    int
    Params::setString( std::string key, std::string val )
    {
        _params[key] = new StringParam( val );
        return EXIT_SUCCESS;
    }

    std::string
    Params::getString(std::string key , std::string default_value) const
    {
        if ( this->hasKey(key) )
        {
            return _params.at(key)->asStringParam()->_value;
        }
        else
        {
            std::cerr << "Params::getString( " << key << ") empty, returning default value " << default_value << std::endl;
            return default_value;
        }
    }

    int
    Params::setDouble( std::string key, double val )
    {
        _params[key] = new DoubleParam( val );
        return EXIT_SUCCESS;
    }

    double
    Params::getDouble( std::string key, double default_value ) const
    {
        if ( this->hasKey(key) )
        {
            return _params.at(key)->asDoubleParam()->_value;
        }
        else
        {
            std::cerr << "Params::getDouble( " << key << ") empty, returning default value " << default_value << std::endl;
            return default_value;
        }
    }

    bool
    Params::hasKey( std::string key ) const
    {
        return _params.find(key) != _params.end();
    }

    Params::~Params()
    {
        for ( auto& param : _params )
        {
            if ( param.second )
            {
                delete param.second;
                param.second = nullptr;
            }
        }
    }

} // nsam
