#include "cuboid_approx/io/walls_io.h"

#include <fstream>
#include <primitives/Cuboid.h>

namespace ca
{
    namespace io
    {
        int
        readWalls( std::map<int, am::Cuboid::TYPE>       & walls
                   , std::string                    const& path
                   , int                            const* cuboid_count
                   , bool                           const  verbose )
        {
            std::ifstream myfile;
            myfile.open( path );
            if ( !myfile.is_open() )
            {
                std::cerr << "[" << __func__ << "]: " << " could not open file " << path << std::endl;
                return EXIT_FAILURE;
            }

            std::string line;
            while ( getline(myfile,line) )
            {
                try
                {
                    std::istringstream iss( line );
                    std::string word;

                    int field_id  = 0;
                    int cuboid_id = -1;
                    while ( std::getline(iss, word, ',') )
                    {
                        switch( field_id )
                        {
                            case 0:
                                if ( verbose ) std::cout << "cuboidword: " << word << " parsed: " << atoi( word.c_str() ) << std::endl;
                                cuboid_id = atoi( word.c_str() );

                                if ( cuboid_count && (cuboid_id >= *cuboid_count) )
                                {
                                    std::cerr << "[" << __func__ << "]: " << "Don't have this many cuboids (" << cuboid_id  << ">= " << *cuboid_count << ")\n"; fflush(stderr);
                                    return WALL_EXCEEDS_CUBOID_COUNT;
                                }
                                break;
                            case 1:
                                if ( verbose ) std::cout << "wallword: " << word << ( word == "w" ? "WALL" : "NOT wall") << std::endl;
                                //_controller->setPrimitiveIsWall( cuboid_id, word == "w", /* selected: */ false );
                                if ( word == "w" ) walls[cuboid_id] = am::Cuboid::WALL;
                                break;
                            case 2:
                                if ( verbose ) std::cout << "floorword: " << word << ( word == "f" ? "FLOOR" : "NOT floor") << std::endl;
                                //_controller->setPrimitiveIsFloor( cuboid_id, word == "f", /* selected: */ false );
                                if ( word == "f" ) walls[cuboid_id] = am::Cuboid::FLOOR;
                                break;
                            case 3:
                                //if ( verbose ) std::cout << "shapeword: " << word << ( strcmp(word,"cub") ? "FLOOR" : "NOT floor") << std::endl;
                                //if ( !strcmp(word.c_str(),"cuboid") ) walls[cuboid_id] = am::Cuboid::CUBOID;
                                    if ( !strcmp(word.c_str(),"sphere") ) walls[cuboid_id] = am::Cuboid::SPHERE;
                                else if ( !strcmp(word.c_str(),"cylinder") ) walls[cuboid_id] = am::Cuboid::CYLINDER;
                                if ( verbose ) std::cout << "shapeword: " << word << am::Cuboid::typeToString( walls[cuboid_id] ) << std::endl;

                                break;
                            default:
                                std::cerr << __func__ << " not prepared for so many wall params" << std::endl;
                                break;
                        } // ... switch field_id

                        ++field_id;
                    } // ... while word in line
                } // ... try
                catch (std::exception ex )
                {
                    std::cerr << __func__ << ex.what() << std::endl;
                    return EXIT_FAILURE;
                } // ... catch
            } // ... while getline

            return EXIT_SUCCESS;
        } // ... read walls

        std::string
        writeWalls( am::Cuboid::TypeMap const& walls
                    , std::string       const& path
                    , int               const  cuboid_count )
        {
            typedef am::Cuboid::TypeMap::const_iterator const_iterator;

            std::ofstream myfile;
            myfile.open ( path );
            if ( !myfile.is_open() )
            {
                std::cerr << "could not open file " << path << std::endl;
                return "EXIT_FAILURE";
            }

            std::string message;
            char tmp[255];
            const const_iterator end_it = walls.end();
            for ( int i = 0; i != cuboid_count; ++i )
            {
                sprintf( tmp, "%02d,", i );
                message += std::string(tmp);

                const_iterator it = walls.find( i );

                message += ((it != end_it) && (it->second == am::Cuboid::WALL)) ? "w," : "n,";
                message += ((it != end_it) && (it->second == am::Cuboid::FLOOR)) ? "f," : "n,";
                message += ((it != end_it)) ? am::Cuboid::typeToString(it->second) : "n,";
                //message += ((it != end_it) && (it->second == am::Cuboid::FLOOR)) ? "f," : "n,";
                message += "\n";
            }

            myfile << message;
            myfile.close();

            std::cout << "walls written to " << path << std::endl;

            return message;
        } // ... write walls
    } // ... namespace io
} // ... namespace ca
