#include "cuboid_approx/mapBack.h"

#if 1
#include <stdio.h>

#include <iostream>
using namespace std;
using namespace ca;

ca::MapBack::MapBack()
{
    // params from Youyi
    _modelViewMat << 1,             0,              0, 0,
                     0,   0.995037146,    0.099503718, 0,
                     0,  -0.099503718,    0.995037146, 0,
                     0,             0,   -1.004987478, 1;

    _projectionMat << 1.119603276,           0,             0,      0,
                                0, 1.732050776,             0,      0,
                                0,           0,  -2.009975195,   -1.0,
                                0,           0,  -1.519999981,      1;

    _viewportMat << 0, 0, 987, 638;
}

int
ca::MapBack::project( Eigen::Matrix<Scalar,3,1> &pnt2D, Eigen::Matrix<Scalar,3,1> const& pnt_arg ) const
{
    return MapBack::project( pnt2D
                             , pnt_arg
                             , ModelViewMat()
                             , ProjectionMat()
                             , ViewportMat() );
}

inline int
ca::MapBack::project( Eigen::Matrix<Scalar,3,1> &pnt2D
                      , Eigen::Matrix<Scalar,3,1> const& pnt_arg
                      , Eigen::Matrix<Scalar,4,4> const& modelview
                      , Eigen::Matrix<Scalar,4,4> const& projection
                      , Eigen::Matrix<Scalar,4,1> const& viewport
                      )
{
    //Transformation vectors
    Scalar fTempo[8];
    //Modelview transform
    fTempo[0]=modelview(0)*pnt_arg(0)+modelview(4)*pnt_arg(1)+modelview(8)*pnt_arg(2)+modelview(12);  //w is always 1
    fTempo[1]=modelview(1)*pnt_arg(0)+modelview(5)*pnt_arg(1)+modelview(9)*pnt_arg(2)+modelview(13);
    fTempo[2]=modelview(2)*pnt_arg(0)+modelview(6)*pnt_arg(1)+modelview(10)*pnt_arg(2)+modelview(14);
    fTempo[3]=modelview(3)*pnt_arg(0)+modelview(7)*pnt_arg(1)+modelview(11)*pnt_arg(2)+modelview(15);
    //Projection transform, the final row of projection matrix is always [0 0 -1 0]
    //so we optimize for that.
    fTempo[4]=projection(0)*fTempo[0]+projection(4)*fTempo[1]+projection(8)*fTempo[2]+projection(12)*fTempo[3];
    fTempo[5]=projection(1)*fTempo[0]+projection(5)*fTempo[1]+projection(9)*fTempo[2]+projection(13)*fTempo[3];
    fTempo[6]=projection(2)*fTempo[0]+projection(6)*fTempo[1]+projection(10)*fTempo[2]+projection(14)*fTempo[3];
    fTempo[7]=-fTempo[2];

    //The result normalizes between -1 and 1
    if ( fTempo[7] == static_cast<Scalar>(.0) )        //The w value
    {
        pnt2D.setZero();
        return EXIT_FAILURE;
    }

    fTempo[7] = 1.0/fTempo[7];
    //Perspective division
    fTempo[4] *= fTempo[7];
    fTempo[5] *= fTempo[7];
    fTempo[6] *= fTempo[7];
//    pnt2D(0) = fTempo[4] * fTempo[7];
//    pnt2D(1) = fTempo[5] * fTempo[7];
//    pnt2D(2) = fTempo[6] * fTempo[7];

    //Window coordinates
    //Map x, y to range 0-1
    //windowCoordinate[0]=(fTempo[4]*0.5+0.5)*viewport[2]+viewport[0];
    //windowCoordinate[1]=(fTempo[5]*0.5+0.5)*viewport[3]+viewport[1];
    //This is only correct when glDepthRange(0.0, 1.0)
    //windowCoordinate[2]=(1.0+fTempo[6])*0.5;  //Between 0 and 1

    pnt2D(0) = (fTempo[4]*0.5+0.5)*viewport(2)+viewport(0);
    pnt2D(1) = (fTempo[5]*0.5+0.5)*viewport(3)+viewport(1);
    // This is only correct when glDepthRange(0.0, 1.0)
    pnt2D(2) = (1.0+fTempo[6])*0.5;  //Between 0 and 1

//    std::cout << "in: " << pnt_arg.transpose()
//              << "\t->\t" << fTempo[4] << ", " << fTempo[5] << ", " << fTempo[6] << ", " << fTempo[7]
//              << "\t->\t" << pnt2D.transpose() << std::endl;

    return EXIT_SUCCESS;
}


int glhProjectf(float objx, float objy, float objz, float *modelview, float *projection, int *viewport, float *windowCoordinate)
{
    //Transformation vectors
    float fTempo[8];
    //Modelview transform
    fTempo[0]=modelview[0]*objx+modelview[4]*objy+modelview[8]*objz+modelview[12];  //w is always 1
    fTempo[1]=modelview[1]*objx+modelview[5]*objy+modelview[9]*objz+modelview[13];
    fTempo[2]=modelview[2]*objx+modelview[6]*objy+modelview[10]*objz+modelview[14];
    fTempo[3]=modelview[3]*objx+modelview[7]*objy+modelview[11]*objz+modelview[15];
    //Projection transform, the final row of projection matrix is always [0 0 -1 0]
    //so we optimize for that.
    fTempo[4]=projection[0]*fTempo[0]+projection[4]*fTempo[1]+projection[8]*fTempo[2]+projection[12]*fTempo[3];
    fTempo[5]=projection[1]*fTempo[0]+projection[5]*fTempo[1]+projection[9]*fTempo[2]+projection[13]*fTempo[3];
    fTempo[6]=projection[2]*fTempo[0]+projection[6]*fTempo[1]+projection[10]*fTempo[2]+projection[14]*fTempo[3];
    fTempo[7]=-fTempo[2];
    //The result normalizes between -1 and 1
    if(fTempo[7]==0.0)        //The w value
        return 0;
    fTempo[7]=1.0/fTempo[7];
    //Perspective division
    fTempo[4]*=fTempo[7];
    fTempo[5]*=fTempo[7];
    fTempo[6]*=fTempo[7];
    //Window coordinates
    //Map x, y to range 0-1
    windowCoordinate[0]=(fTempo[4]*0.5+0.5)*viewport[2]+viewport[0];
    windowCoordinate[1]=(fTempo[5]*0.5+0.5)*viewport[3]+viewport[1];
    //This is only correct when glDepthRange(0.0, 1.0)
    windowCoordinate[2]=(1.0+fTempo[6])*0.5;  //Between 0 and 1
    return 1;
}

//int main()
//{
//    Eigen::Matrix<float,3,1> pnt; pnt << 0.25f,0.33f,3.0f;
//    Eigen::Matrix<float,3,1> pnt2D;
//    MapBack mapBack;
//    int err = MapBack::project( pnt2D, pnt, mapBack.ModelViewMat(), mapBack.ProjectionMat(), mapBack.ViewportMat() );
//    std::cout << "pnt: " << pnt.transpose() << " -> pnt2D: " << pnt2D.transpose() << std::endl;

//    std::cout << "ok" << std::endl;

//    return err;
//}

#endif
