#ifndef FURNITURE_SUITE_H
#define FURNITURE_SUITE_H
/*
 * =====================================================================================
 *
 *       Filename:  furniture.h
 *
 *    Description:  Class header for main furniture suite
 * 
 *        Version:  1.0
 *        Created:  10/01/2013 03:27:05 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *         Author:  Moos Hueting (mh), mooshueting@gmail.com
 *   Organization:  
 *
 * =====================================================================================
 */
#include <pcl/common/common.h>
#include <pcl/point_types.h>
#include <pcl/io/pcd_io.h>
#include <pcl/filters/extract_indices.h>
#include <pcl/filters/project_inliers.h>
#include <pcl/filters/voxel_grid.h>
#include <pcl/filters/statistical_outlier_removal.h>
#include <pcl/kdtree/kdtree.h>
#include <pcl/sample_consensus/method_types.h>
#include <pcl/sample_consensus/model_types.h>
#include <pcl/segmentation/sac_segmentation.h>
#include <pcl/segmentation/extract_clusters.h>
#include <pcl/visualization/pcl_visualizer.h>

#include <iostream>
#include <vector>
#include <string>

typedef pcl::PointCloud<pcl::PointXYZRGB> PointCloud;

class FurnitureSuite {
        boost::shared_ptr<pcl::visualization::PCLVisualizer> viewer;
        PointCloud::Ptr cloud; 

        pcl::ModelCoefficients::Ptr groundPlane;

        std::vector<pcl::ModelCoefficients> planeCoefficients;
        std::vector<pcl::PointIndices> clusterIndices;

        void _selectGroundPlane();
        void _drawClusterBoundingBoxes();
    public:
        FurnitureSuite();
        FurnitureSuite(std::string);
        FurnitureSuite(PointCloud::Ptr _cloud) : cloud(_cloud) {};

        // load and save point cloud from file
        void loadPointCloud(std::string);
        void savePointCloud(std::string);

        PointCloud::Ptr getCloud();

        // different filters
        void removeStatisticalNoise();
        void decimatePointCloud();

        // detect planes in point cloud, such as wall and floor
        void detectPlanes();

        // segment point cloud into objects
        void segment();

        // show elements in viewer
        void showPointCloud();
        void showPlanes();
        void showClusters();

        // show viewer
        void showViewer();

};
#endif
