#include "Primitive.h"

#include "boost/make_shared.hpp"
#include "pcl/filters/extract_indices.h"
#include "pcl/common/io.h"

namespace am
{
    Primitive::Primitive()
        : _indicesChanged( true )
        , _roi_cloud_ptr( new PCloud )
    {

    }

    Primitive::Primitive( PCloud::ConstPtr cloud_ptr )
        : _cloud_ptr( cloud_ptr )
        , _roi_cloud_ptr( new PCloud )
        , _indicesChanged( true )
    {

    }

    int
    Primitive::unDraw( pcl::visualization::PCLVisualizer::Ptr v, std::string cloud_name )
    {
        std::cerr << "Primitive::unDraw() has to be overridden" << std::endl;
        return EXIT_FAILURE;
    }

    int Primitive::cloneTo( Primitive &ret ) const
    {
        if ( _cloud_ptr )
        {
            pcl::PointCloud<MyPoint>::Ptr tmp_cloud( new PCloud() );
            tmp_cloud->reserve( _cloud_ptr->size() );
            for ( size_t pid = 0; pid != _cloud_ptr->size(); ++pid )
                tmp_cloud->push_back( _cloud_ptr->at(pid) );
            ret._cloud_ptr = tmp_cloud;
        }

        if ( _indices_ptr )
        {
            ret._indices_ptr.reset( new pcl::PointIndices() );
            ret._indices_ptr->indices = _indices_ptr->indices;
        }

        ret._indicesChanged = true;

        return EXIT_SUCCESS;
    }

    PCloud::Ptr const&
    Primitive::getROICloud() const
    {
        //PCloud::ConstPtr cloud_ptr( new PCloud );

        if ( !_indices_ptr->indices.size() )
        {
            std::cerr << "Primitive::getROICloud does not have indices yet, returning whole cloud" << std::endl;
            //PCloud::Ptr tmp( new PCloud );
            //pcl::copyPointCloud( *const_cast<PCloud::Ptr>(_cloud_ptr), *tmp );
            pcl::copyPointCloud( *_cloud_ptr, *_roi_cloud_ptr );
            return _roi_cloud_ptr;
            //return tmp;
        }

        if ( _indicesChanged )
        {
            pcl::ExtractIndices<MyPoint> extraction;
            PCloud::Ptr tmp_cloud( new PCloud );
            {
                extraction.setInputCloud( _cloud_ptr );
                extraction.setIndices   ( _indices_ptr );
                extraction.setNegative  ( false );
                extraction.filter       ( *tmp_cloud );
                _roi_cloud_ptr = tmp_cloud;
            }
            _indicesChanged = false;
        }

        return _roi_cloud_ptr;
    }

    // fetches const pointer and creates deep copy
    int
    Primitive::cloneROICloud( PCloud::Ptr &cloud_ptr )
    {
        if ( !cloud_ptr ) cloud_ptr = boost::make_shared<PCloud>();

        pcl::copyPointCloud( *this->getROICloud(), *cloud_ptr );

        return EXIT_SUCCESS;
    }


    PCloud::ConstPtr
    Primitive::CloudPtr() const
    {
        return _cloud_ptr;
    }

    PCloud::ConstPtr&
    Primitive::CloudPtrRef()
    {
        return _cloud_ptr;
    }

    /*PCloud::Ptr
    Primitive::CloudNonConstPtr() const
    {
        return _cloud_ptr;
    }*/

    const pcl::PointIndices::Ptr
    Primitive::IndicesPtr() const
    {
        return _indices_ptr;
    }

    pcl::PointIndices::Ptr&
    Primitive::IndicesPtrRef()
    {
        _indicesChanged = true;
        return _indices_ptr;
    }

    pcl::PointIndices::Ptr
    Primitive::IndicesNonConstPtr()
    {
        _indicesChanged = true;
        return _indices_ptr;
    }

} // ns am
