#include "Plane.h"

#include <vector>

#include "pcl/common/transforms.h"          // transformPointCloud
#include "pcl/common/common.h"              // getMinMax3D
#include "pcl/filters/project_inliers.h"    // projectInliers
#include "pcl/common/centroid.h"            // compute3DCentroid
#include "pcl/segmentation/sac_segmentation.h" // sacsegmentation
#include "pcl/filters/filter_indices.h"
#include "pcl/correspondence.h"
#include "pcl/io/pcd_io.h"

#include "cuboid_approx/util/smartgeometry/pcltools/util.hpp"
#include "cuboid_approx/util/AMUtil2.h"
#include "CloudViewer.h"

#include "Segmentation.h"
#include "DepthProcessor.h"
#include "Cuboid.h"

namespace am
{
#if 0
    template <typename PointT>
    int clusterNYU( boost::shared_ptr<pcl::PointCloud<PointT> > cloud
                    , boost::shared_ptr<pcl::PointIndices > indices_arg
                    , int   const K             = 15
                    , float const radius        = -1.f
                    , int   const filter_nmeans = 50
                    , float const filter_stddev = .1
                    , float const density_cap_median_multiplier = 1.5f
                    , bool  const debug         = false )
    {
        using std::vector;

        // cleanup
        //typename pcl::PointCloud<PointT>::Ptr cloud( new typename pcl::PointCloud<PointT>() );
        if ( indices_arg ) std::cout << "in_indices: " << indices_arg->indices.size() << std::endl;
        pcl::PointIndices::Ptr indices( new pcl::PointIndices() );
        am::DepthProcessor::statistical_clean<PointT>( indices
                                                       , cloud
                                                       , indices_arg ? &indices_arg : nullptr
                                                       , nullptr
                                                       , filter_nmeans
                                                       , filter_stddev );
        if ( indices_arg ) std::cout << "out_indices: " << indices->indices.size() << std::endl;

        // debug
        boost::shared_ptr<am::CloudViewer> vptr;
        if ( debug ) vptr = am::CloudViewer::showCloud( cloud, "cloud", indices, 2., true );

        /// mean radius
        pcl::PointIndices::Ptr roi_indices( new pcl::PointIndices() );
        vector<vector<int>   > neighbour_indices;
        vector<float         > densities;
        std::pair<float,float> min_max_density( FLT_MAX, 0.f );
        float                  avg_density = 0.f;
        {
            // neighbours
            vector<vector<float> > neighbour_dists;
            smartgeometry::getNeighbourhoodIndices( /* neighbours: */ neighbour_indices
                                                    , /*    cloud: */ cloud
                                                    , /*  indices: */ &(indices->indices)
                                                    , /*    dists: */ &neighbour_dists
                                                    , /*   max_ns: */ K
                                                    , /*   radius: */ radius );
            // calc densitites
            densities.resize( neighbour_indices.size(), 0.f );
            for ( size_t pid = 0; pid != neighbour_indices.size(); ++pid )
            {
                for ( size_t nid = 0; nid != neighbour_indices[pid].size(); ++nid )
                    densities[pid] += sqrt( neighbour_dists[pid][nid] );
                densities[pid] /= static_cast<float>(neighbour_indices[pid].size()) / 1000.f;

                // min, max, avg
                if ( densities[pid] < min_max_density.first  ) min_max_density.first  = densities[pid];
                if ( densities[pid] > min_max_density.second ) min_max_density.second = densities[pid];
                avg_density += densities[pid];
            }
            // avg
            avg_density /= neighbour_indices.size();

            if ( debug )
            {
                std::string   fname = "densities.txt";
                std::ofstream f( fname );

                // colour output
                //const float span = min_max_density.second - min_max_density.first;
                for ( size_t iid = 0; iid != neighbour_indices.size(); ++iid )
                {
                    // float ratio = (densities[iid] - min_max_density.first) / span;
                    am::util::pcl::setPointColor( cloud->at( indices->indices[iid] )
                                                  , std::max(0.f,(10.f - densities[iid]))*255.f, 0.f, 0.f );
                    f << cloud->at(indices->indices[iid]).z << " " << densities[iid] << "\n";
                }
                f.close();
                // std::cout << "calling " << ("gnuplot -e \"plot '" + fname + "' u 1:2 w points\" -p") << std::endl;
                // system( ("gnuplot -e \"plot '" + fname + "' u 1:2 w points\" -p").c_str() );
            }
        }

        if ( debug )
        {
            vptr->VP()->setCameraPosition( 0,-4,2, 0,0,3, 0,-1,0 );
            std::cerr << "avg: " << avg_density << std::endl;
            std::cerr << "median: " << am::util::algo::median( densities ) << std::endl;
        }

        am::Cuboid cuboid;
        float cap = am::util::algo::median( densities ) * density_cap_median_multiplier;
        std::cout << "cap: " << cap << std::endl;
        {
            roi_indices->indices.clear();

            for ( size_t iid = 0; iid != neighbour_indices.size(); ++iid )
            {
                if ( densities[iid] < cap )
                {
                    roi_indices->indices.push_back( indices->indices[iid] );

                    if ( debug )
                        am::util::pcl::setPointColor( cloud->at(indices->indices[iid])
                                                      , std::max(0.f,(cap - densities[iid]))*255.f, 0.f, 0.f );
                }
            }
            std::cout << "roi_indices->size: " << roi_indices->indices.size() << std::endl;

            // PCA
            {
                Eigen::Matrix4f frame;
                am::DepthProcessor::PCA( frame
                                         , cloud
                                         , roi_indices );
                am::Cuboid::fromFrame( cuboid
                                       , frame
                                       , cloud
                                       , roi_indices );

                if ( debug )
                {
                    vptr->VP()->removeAllShapes();
                    cuboid.draw( vptr->VP()
                                 , "cuboid"
                                 , (Eigen::Vector3f() << .9, .0, .9).finished()
                                 , 5.
                                 , true );

                }
            }

            if ( debug )
            {
                am::CloudViewer::updatePointCloud( vptr->VP(), cloud, "cloud", indices );
                vptr->VP()->setPointCloudRenderingProperties( pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 2.5, "cloud", 0 );
                vptr->VP()->spin();
            }
        }

        //cuboid.IndicesPtrRef()->indices = am::util::pcl::allIndicesOf( cloud )->indices;
        cuboid.IndicesPtrRef()->indices = indices->indices;
        am::Cuboid cuboid2;
        Cuboid::clampPointsToOrientedPlanes( cuboid2, cuboid, cuboid.Planes(), -FLT_EPSILON );

        if ( debug )
        {
            for ( size_t pid = 0; pid != cuboid.IndicesPtr()->indices.size(); ++pid )
                am::util::pcl::setPointColor( cloud->at(cuboid.IndicesPtr()->indices[pid]), 0.f, 255.f, 255.f );
            for ( size_t pid = 0; pid != cuboid2.IndicesPtr()->indices.size(); ++pid )
                am::util::pcl::setPointColor( cloud->at(cuboid2.IndicesPtr()->indices[pid]), 255.f, 255.f, 0.f );
        }

        // PCA
        {
            Eigen::Matrix4f frame;
            am::DepthProcessor::PCA( frame
                                     , cuboid2.CloudPtr()
                                     , cuboid2.IndicesPtr() );

            am::Cuboid::fromFrame( cuboid2
                                   , frame
                                   , cuboid2.CloudPtr()
                                   , cuboid2.IndicesPtr() );

            if ( debug )
            {
                vptr->VP()->removeAllShapes();
                cuboid2.draw( vptr->VP(), "cuboid2" );
            }
        }

        if ( debug )
        {
            am::CloudViewer::updatePointCloud( vptr->VP(), cloud, "cloud", indices );
            vptr->VP()->setPointCloudRenderingProperties( pcl::visualization::PCL_VISUALIZER_POINT_SIZE, 2.5, "cloud", 0 );
            vptr->VP()->spin();
        }

        return EXIT_SUCCESS;
    }
#endif
}

