#include "cuboid_approx/ui/DisplayedPrimitive.h"

#include "primitives/Cuboid.h"
#include "primitives/Plane.h"
#include "AnnotationController.h"

namespace ca
{
    using am::AnnotationController;

    /// ------------------------------------------------------------------------------------------------ ///
    /// CLASS DisplayedPrimitive
    /// ------------------------------------------------------------------------------------------------ ///
    DisplayedPrimitive::DisplayedPrimitive( am::Cuboid const& cuboid )
        : primitive_ptr( new am::Cuboid(cuboid) )
        , hidden(false)
        , wall(false)
        , floor(false)
        , rotation_target(false)
        , tag            ("")
        , shape( am::Cuboid::TYPE::CUBOID )
    {

    }

    DisplayedPrimitive::DisplayedPrimitive( am::Plane const& plane )
        : primitive_ptr( new am::Plane( plane ) )
        , hidden(false)
        , wall(false)
        , floor(false)
        , rotation_target(false)
        , tag            ("")
        , shape( am::Cuboid::TYPE::CUBOID )
    {

    }

    std::shared_ptr<am::Cuboid>
    DisplayedPrimitive::CuboidPtr() const
    {
        return std::static_pointer_cast< am::Cuboid,am::Primitive>( primitive_ptr );
    }

    std::shared_ptr<am::Plane>
    DisplayedPrimitive::PlanePtr() const
    {
        return std::static_pointer_cast< am::Plane,am::Primitive>( primitive_ptr );
    }

    Eigen::Vector4f
    DisplayedPrimitive::getDrawingColour( bool colour_code_cuboids ) const
    {
        Eigen::Vector4f ret = colour_code_cuboids ? this->colour : AnnotationController::CUBOID_COLOUR;
        if ( wall  )   ret = AnnotationController::CUBOID_WALL_COLOUR;
        else if ( floor )   ret = AnnotationController::CUBOID_FLOOR_COLOUR;

        ret(3) = this->colour(3);

        return ret;
    }

    int
    DisplayedPrimitive::reDraw( pcl::visualization::PCLVisualizer::Ptr viewer
                               , std::string                      const name
                               , bool                             const selected
                               , bool                             const colour_code_cuboids
                               , bool                             const colour_edges
                               , bool                             const show_ids )
                               //, am::Primitive::Shape             const shape               /* = CUBOID */ )
    {
        this->CuboidPtr()->unDraw( viewer, name );
        //auto shapemap = viewer->getShapeActorMap();
//        for ( auto it = shapemap->begin(); it != shapemap->end(); ++it )
//        {
//            std::cout << "shape " << it->first << std::endl;
//        }

        auto current_colour = this->getDrawingColour( colour_code_cuboids );

        return this->CuboidPtr()->draw4( viewer
                                         , name
                                         , current_colour
                                         , selected ? AnnotationController::CUBOID_SELECTED_LINE_WIDTH
                                                    : AnnotationController::CUBOID_UNSELECTED_LINE_WIDTH
                                         , colour_edges
                                         , /* showName: */ show_ids
                                         , /*    Shape: */ shape );
    }

} // ... ns ca
